@@ -42,9 +42,12 @@ class auth_plugin_mnet extends auth_plugin_base {
     }
 
     /**
-     * Old syntax of class constructor for backward compatibility.
+     * Old syntax of class constructor. Deprecated in PHP7.
+     *
+     * @deprecated since Moodle 3.1
      */
     public function auth_plugin_mnet() {
+        debugging('Use of class name as constructor is deprecated', DEBUG_DEVELOPER);
         self::__construct();
     }
 
@@ -65,7 +68,7 @@ class auth_plugin_mnet extends auth_plugin_base {
      * Return user data for the provided token, compare with user_agent string.
      *
      * @param  string $token    The unique ID provided by remotehost.
-     * @param  string $UA       User Agent string.
+     * @param  string $useragent       User Agent string.
      * @return array  $userdata Array of user info for remote host
      */
     function user_authorise($token, $useragent) {
@@ -260,7 +263,7 @@ class auth_plugin_mnet extends auth_plugin_base {
             exit;
         }
 
-        if (user_not_fully_set_up($remoteuser)) {
+        if (user_not_fully_set_up($remoteuser, false)) {
             print_error('notenoughidpinfo', 'mnet');
             exit;
         }
@@ -275,13 +278,7 @@ class auth_plugin_mnet extends auth_plugin_base {
         if (isset($remoteuser->lang)) {
             $remoteuser->lang = clean_param(str_replace('_utf8', '', $remoteuser->lang), PARAM_LANG);
         }
-        if (empty($remoteuser->lang)) {
-            if (!empty($CFG->lang)) {
-                $remoteuser->lang = $CFG->lang;
-            } else {
-                $remoteuser->lang = 'en';
-            }
-        }
+
         $firsttime = false;
 
         // get the local record for the remote user
@@ -299,7 +296,7 @@ class auth_plugin_mnet extends auth_plugin_base {
             $remoteuser->firstaccess = 0;
             $remoteuser->confirmed = 1;
 
-            $remoteuser->id = $DB->insert_record('user', $remoteuser);
+            $remoteuser->id = user_create_user($remoteuser, false);
             $firsttime = true;
             $localuser = $remoteuser;
         }
@@ -379,7 +376,7 @@ class auth_plugin_mnet extends auth_plugin_base {
             // with info so that the IDP can maintain mnetservice_enrol_enrolments
             $mnetrequest->add_param($remoteuser->username);
             $fields = 'id, category, sortorder, fullname, shortname, idnumber, summary, startdate, visible';
-            $courses = enrol_get_users_courses($localuser->id, false, $fields, 'visible DESC,sortorder ASC');
+            $courses = enrol_get_users_courses($localuser->id, false, $fields);
             if (is_array($courses) && !empty($courses)) {
                 // Second request to do the JOINs that we'd have done
                 // inside enrol_get_users_courses() if we had been allowed
@@ -613,98 +610,6 @@ class auth_plugin_mnet extends auth_plugin_base {
         return null;
     }
 
-    /**
-     * Prints a form for configuring this authentication plugin.
-     *
-     * This function is called from admin/auth.php, and outputs a full page with
-     * a form for configuring this plugin.
-     *
-     * @param object $config
-     * @param object $err
-     * @param array $user_fields
-     */
-    function config_form($config, $err, $user_fields) {
-        global $CFG, $DB;
-
-         $query = "
-            SELECT
-                h.id,
-                h.name as hostname,
-                h.wwwroot,
-                h2idp.publish as idppublish,
-                h2idp.subscribe as idpsubscribe,
-                idp.name as idpname,
-                h2sp.publish as sppublish,
-                h2sp.subscribe as spsubscribe,
-                sp.name as spname
-            FROM
-                {mnet_host} h
-            LEFT JOIN
-                {mnet_host2service} h2idp
-            ON
-               (h.id = h2idp.hostid AND
-               (h2idp.publish = 1 OR
-                h2idp.subscribe = 1))
-            INNER JOIN
-                {mnet_service} idp
-            ON
-               (h2idp.serviceid = idp.id AND
-                idp.name = 'sso_idp')
-            LEFT JOIN
-                {mnet_host2service} h2sp
-            ON
-               (h.id = h2sp.hostid AND
-               (h2sp.publish = 1 OR
-                h2sp.subscribe = 1))
-            INNER JOIN
-                {mnet_service} sp
-            ON
-               (h2sp.serviceid = sp.id AND
-                sp.name = 'sso_sp')
-            WHERE
-               ((h2idp.publish = 1 AND h2sp.subscribe = 1) OR
-               (h2sp.publish = 1 AND h2idp.subscribe = 1)) AND
-                h.id != ?
-            ORDER BY
-                h.name ASC";
-
-        $id_providers       = array();
-        $service_providers  = array();
-        if ($resultset = $DB->get_records_sql($query, array($CFG->mnet_localhost_id))) {
-            foreach($resultset as $hostservice) {
-                if(!empty($hostservice->idppublish) && !empty($hostservice->spsubscribe)) {
-                    $service_providers[]= array('id' => $hostservice->id, 'name' => $hostservice->hostname, 'wwwroot' => $hostservice->wwwroot);
-                }
-                if(!empty($hostservice->idpsubscribe) && !empty($hostservice->sppublish)) {
-                    $id_providers[]= array('id' => $hostservice->id, 'name' => $hostservice->hostname, 'wwwroot' => $hostservice->wwwroot);
-                }
-            }
-        }
-
-        include "config.html";
-    }
-
-    /**
-     * Processes and stores configuration data for this authentication plugin.
-     */
-    function process_config($config) {
-        // set to defaults if undefined
-        if (!isset ($config->rpc_negotiation_timeout)) {
-            $config->rpc_negotiation_timeout = '30';
-        }
-        /*
-        if (!isset ($config->auto_add_remote_users)) {
-            $config->auto_add_remote_users = '0';
-        } See MDL-21327   for why this is commented out
-        set_config('auto_add_remote_users',   $config->auto_add_remote_users,   'auth_mnet');
-        */
-
-        // save settings
-        set_config('rpc_negotiation_timeout', $config->rpc_negotiation_timeout, 'auth_mnet');
-
-        return true;
-    }
-
     /**
      * Poll the IdP server to let it know that a user it has authenticated is still
      * online
@@ -805,9 +710,7 @@ class auth_plugin_mnet extends auth_plugin_base {
 
         foreach($superArray as $subArray) {
             $subArray = array_values($subArray);
-            $instring = "('".implode("', '",$subArray)."')";
-            $query = "select id, session_id, username from {mnet_session} where username in $instring";
-            $results = $DB->get_records_sql($query);
+            $results = $DB->get_records_list('mnet_session', 'username', $subArray, '', 'id, session_id, username');
 
             if ($results == false) {
                 // We seem to have a username that breaks our query:
@@ -826,25 +729,6 @@ class auth_plugin_mnet extends auth_plugin_base {
         return array('code' => 1, 'message' => $returnString, 'last log id' => $remoteclient->last_log_id);
     }
 
-    /**
-     * Cron function will be called automatically by cron.php every 5 minutes
-     *
-     * @return void
-     */
-    function cron() {
-        global $DB;
-
-        // run the keepalive client
-        $this->keepalive_client();
-
-        $random100 = rand(0,100);
-        if ($random100 < 10) {     // Approximately 10% of the time.
-            // nuke olden sessions
-            $longtime = time() - (1 * 3600 * 24);
-            $DB->delete_records_select('mnet_session', "expires < ?", array($longtime));
-        }
-    }
-
     /**
      * Cleanup any remote mnet_sessions, kill the local mnet_session data
      *
@@ -1028,7 +912,7 @@ class auth_plugin_mnet extends auth_plugin_base {
      *
      * @see process_new_icon()
      * @uses mnet_remote_client callable via MNet XML-RPC
-     * @param int $userid The id of the user
+     * @param int $username The id of the user
      * @return false|array false if user not found, empty array if no picture exists, array with data otherwise
      */
     function fetch_user_image($username) {
@@ -1153,25 +1037,16 @@ class auth_plugin_mnet extends auth_plugin_base {
     }
 
     /**
-     * Returns a list of potential IdPs that this authentication plugin supports.
-     * This is used to provide links on the login page.
+     * Returns a list of MNet IdPs that the user can roam from.
      *
-     * @param string $wantsurl the relative url fragment the user wants to get to.  You can use this to compose a returnurl, for example
-     *
-     * @return array like:
-     *              array(
-     *                  array(
-     *                      'url' => 'http://someurl',
-     *                      'icon' => new pix_icon(...),
-     *                      'name' => get_string('somename', 'auth_yourplugin'),
-     *                 ),
-     *             )
+     * @param string $wantsurl The relative url fragment the user wants to get to.
+     * @return array List of arrays with keys url, icon and name.
      */
     function loginpage_idp_list($wantsurl) {
         global $DB, $CFG;
 
         // strip off wwwroot, since the remote site will prefix it's return url with this
-        $wantsurl = preg_replace('/(' . preg_quote($CFG->wwwroot, '/') . '|' . preg_quote($CFG->httpswwwroot, '/') . ')/', '', $wantsurl);
+        $wantsurl = preg_replace('/(' . preg_quote($CFG->wwwroot, '/') . ')/', '', $wantsurl);
 
         $sql = "SELECT DISTINCT h.id, h.wwwroot, h.name, a.sso_jump_url, a.name as application
                   FROM {mnet_host} h
@@ -1200,4 +1075,120 @@ class auth_plugin_mnet extends auth_plugin_base {
         }
         return $idps;
     }
+
+    /**
+     * Test if settings are correct, print info to output.
+     */
+    public function test_settings() {
+        global $CFG, $OUTPUT, $DB;
+
+        // Generate warning if MNET is disabled.
+        if (empty($CFG->mnet_dispatcher_mode) || $CFG->mnet_dispatcher_mode !== 'strict') {
+                echo $OUTPUT->notification(get_string('mnetdisabled', 'mnet'), 'notifyproblem');
+                return;
+        }
+
+        // Generate full list of ID and service providers.
+        $query = "
+           SELECT
+               h.id,
+               h.name as hostname,
+               h.wwwroot,
+               h2idp.publish as idppublish,
+               h2idp.subscribe as idpsubscribe,
+               idp.name as idpname,
+               h2sp.publish as sppublish,
+               h2sp.subscribe as spsubscribe,
+               sp.name as spname
+           FROM
+               {mnet_host} h
+           LEFT JOIN
+               {mnet_host2service} h2idp
+           ON
+              (h.id = h2idp.hostid AND
+              (h2idp.publish = 1 OR
+               h2idp.subscribe = 1))
+           INNER JOIN
+               {mnet_service} idp
+           ON
+              (h2idp.serviceid = idp.id AND
+               idp.name = 'sso_idp')
+           LEFT JOIN
+               {mnet_host2service} h2sp
+           ON
+              (h.id = h2sp.hostid AND
+              (h2sp.publish = 1 OR
+               h2sp.subscribe = 1))
+           INNER JOIN
+               {mnet_service} sp
+           ON
+              (h2sp.serviceid = sp.id AND
+               sp.name = 'sso_sp')
+           WHERE
+              ((h2idp.publish = 1 AND h2sp.subscribe = 1) OR
+              (h2sp.publish = 1 AND h2idp.subscribe = 1)) AND
+               h.id != ?
+           ORDER BY
+               h.name ASC";
+
+        $idproviders = array();
+        $serviceproviders = array();
+        if ($resultset = $DB->get_records_sql($query, array($CFG->mnet_localhost_id))) {
+            foreach ($resultset as $hostservice) {
+                if (!empty($hostservice->idppublish) && !empty($hostservice->spsubscribe)) {
+                    $serviceproviders[] = array('id' => $hostservice->id,
+                        'name' => $hostservice->hostname,
+                        'wwwroot' => $hostservice->wwwroot);
+                }
+                if (!empty($hostservice->idpsubscribe) && !empty($hostservice->sppublish)) {
+                    $idproviders[] = array('id' => $hostservice->id,
+                        'name' => $hostservice->hostname,
+                        'wwwroot' => $hostservice->wwwroot);
+                }
+            }
+        }
+
+        // ID Providers.
+        $table = html_writer::start_tag('table', array('class' => 'generaltable'));
+
+        $count = 0;
+        foreach ($idproviders as $host) {
+            $table .= html_writer::start_tag('tr');
+            $table .= html_writer::start_tag('td');
+            $table .= $host['name'];
+            $table .= html_writer::end_tag('td');
+            $table .= html_writer::start_tag('td');
+            $table .= $host['wwwroot'];
+            $table .= html_writer::end_tag('td');
+            $table .= html_writer::end_tag('tr');
+            $count++;
+        }
+            $table .= html_writer::end_tag('table');
+
+        if ($count > 0) {
+            echo html_writer::tag('h3', get_string('auth_mnet_roamin', 'auth_mnet'));
+            echo $table;
+        }
+
+        // Service Providers.
+        unset($table);
+        $table = html_writer::start_tag('table', array('class' => 'generaltable'));
+        $count = 0;
+        foreach ($serviceproviders as $host) {
+            $table .= html_writer::start_tag('tr');
+            $table .= html_writer::start_tag('td');
+            $table .= $host['name'];
+            $table .= html_writer::end_tag('td');
+            $table .= html_writer::start_tag('td');
+            $table .= $host['wwwroot'];
+            $table .= html_writer::end_tag('td');
+            $table .= html_writer::end_tag('tr');
+            $count++;
+        }
+            $table .= html_writer::end_tag('table');
+        if ($count > 0) {
+            echo html_writer::tag('h3', get_string('auth_mnet_roamout', 'auth_mnet'));
+            echo $table;
+        }
+    }
 }
