@@ -15,16 +15,15 @@
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
 /**
- * Version details
+ * Version details.
  *
- * @package    format
- * @subpackage topics
+ * @package    format_topics
  * @copyright  1999 onwards Martin Dougiamas (http://dougiamas.com)
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
 defined('MOODLE_INTERNAL') || die();
 
-$plugin->version   = 2011120100;        // The current plugin version (Date: YYYYMMDDXX)
-$plugin->requires  = 2011120100;        // Requires this Moodle version
-$plugin->component = 'format_topics';    // Full name of the plugin (used for diagnostics)
+$plugin->version   = 2021051700;        // The current plugin version (Date: YYYYMMDDXX).
+$plugin->requires  = 2021051100;        // Requires this Moodle version.
+$plugin->component = 'format_topics';    // Full name of the plugin (used for diagnostics).
