@@ -50,23 +50,10 @@ class helper {
      */
     public static function save_h5p(factory $factory, \stored_file $file, \stdClass $config, bool $onlyupdatelibs = false,
             bool $skipcontent = false) {
-        // This may take a long time.
-        \core_php_time_limit::raise();
 
-        $core = $factory->get_core();
-        $core->h5pF->set_file($file);
-        $path = $core->fs->getTmpPath();
-        $core->h5pF->getUploadedH5pFolderPath($path);
-        // Add manually the extension to the file to avoid the validation fails.
-        $path .= '.h5p';
-        $core->h5pF->getUploadedH5pPath($path);
-
-        // Copy the .h5p file to the temporary folder.
-        $file->copy_content_to($path);
-
-        // Check if the h5p file is valid before saving it.
-        $h5pvalidator = $factory->get_validator();
-        if ($h5pvalidator->isValidPackage($skipcontent, $onlyupdatelibs)) {
+        if (api::is_valid_package($file, $onlyupdatelibs, $skipcontent, $factory, false)) {
+            $core = $factory->get_core();
+            $h5pvalidator = $factory->get_validator();
             $h5pstorage = $factory->get_storage();
 
             $content = [
@@ -83,6 +70,7 @@ class helper {
 
             return $h5pstorage->contentId;
         }
+
         return false;
     }
 
@@ -346,7 +334,7 @@ class helper {
         $usersettings = [];
         if (isloggedin()) {
             $usersettings['name'] = fullname($USER, has_capability('moodle/site:viewfullnames', $systemcontext));
-            $usersettings['mail'] = $USER->email;
+            $usersettings['id'] = $USER->id;
         }
         $settings = array(
             'baseUrl' => $basepath,
