@@ -36,7 +36,7 @@ $string['asyncemailenable'] = 'Enable message notifications';
 $string['asyncemailenabledetail'] = 'If enabled, users will receive a message when an asynchronous backup or restore completes.';
 $string['asyncmessagebody'] = 'Message';
 $string['asyncmessagebodydetail'] = 'Message to send when an asynchronous backup or restore completes.';
-$string['asyncmessagebodydefault'] = 'Hi {user_firstname},<br/> Your {operation} (ID: {backupid}) has completed successfully. <br/><br/>You can access it here: {link}.';
+$string['asyncmessagebodydefault'] = 'Your {operation} (ID: {backupid}) has completed successfully. You can access it here: <a href="{link}">{link}</a>.';
 $string['asyncmessagesubject'] = 'Subject';
 $string['asyncmessagesubjectdetail'] = 'Message subject';
 $string['asyncmessagesubjectdefault'] = 'Moodle {operation} completed successfully';
@@ -172,7 +172,7 @@ $string['copycoursetitle'] = 'Copy course: {$a}';
 $string['copydest'] = 'Destination';
 $string['copyingcourse'] = 'Course copying in progress';
 $string['copyingcourseshortname'] = 'copying';
-$string['copyfieldnotfound'] = 'A required field was not found';
+$string['copyfieldnotfound'] = 'Required field data was not found for field(s): {$a}';
 $string['copyformfail'] = 'AJAX submission of course copy form has failed.';
 $string['copyop'] = 'Current operation';
 $string['copyprogressheading'] = 'Course copies in progress';
@@ -372,7 +372,7 @@ $string['rootsettinggroups'] = 'Include groups and groupings';
 $string['rootsettingimscc1'] = 'Convert to IMS Common Cartridge 1.0';
 $string['rootsettingimscc11'] = 'Convert to IMS Common Cartridge 1.1';
 $string['samesitenotification'] = 'This backup was created with only references to files, not the files themselves. Restoring will only work on this site.';
-$string['sitecourseformatwarning'] = 'This is a front page backup, note that they can only be restored on the front page';
+$string['sitecourseformatwarning'] = 'This is a front page backup. It can only be restored on the front page.';
 $string['storagecourseonly'] = 'Course backup filearea';
 $string['storagecourseandexternal'] = 'Course backup filearea and the specified directory';
 $string['storageexternalonly'] = 'Specified directory for automated backups';
