@@ -153,11 +153,14 @@ $string['confirmunassignno'] = 'Cancel';
 $string['contentbank:access'] = 'Access the content bank';
 $string['contentbank:deleteanycontent'] = 'Delete any content from the content bank';
 $string['contentbank:deleteowncontent'] = 'Delete content from own content bank';
+$string['contentbank:downloadcontent'] = 'Download content from the content bank';
 $string['contentbank:manageanycontent'] = 'Manage any content from the content bank';
 $string['contentbank:manageowncontent'] = 'Manage content from own content bank';
+$string['contentbank:viewunlistedcontent'] = 'View unlisted content from the content bank';
 $string['contentbank:upload'] = 'Upload new content to the content bank';
 $string['contentbank:useeditor'] = 'Create or edit content using a content type editor';
 $string['context'] = 'Context';
+$string['contextrolenotallowed'] = 'Role {$a} not allowed in this context.';
 $string['course:activityvisibility'] = 'Hide/show activities';
 $string['course:bulkmessaging'] = 'Send a message to many people';
 $string['course:create'] = 'Create courses';
@@ -171,6 +174,8 @@ $string['course:changelockedcustomfields'] = 'Change locked custom fields';
 $string['course:changeshortname'] = 'Change course short name';
 $string['course:changesummary'] = 'Change course summary';
 $string['course:configurecustomfields'] = 'Configure custom fields';
+$string['course:configuredownloadcontent'] = 'Configure download course content';
+$string['course:downloadcoursecontent'] = 'Download course content';
 $string['course:enrolconfig'] = 'Configure enrol instances in courses';
 $string['course:enrolreview'] = 'Review course enrolments';
 $string['course:setforcedlanguage'] = 'Force course language';
@@ -247,9 +252,9 @@ $string['extusers'] = 'Existing users';
 $string['extusersmatching'] = 'Existing users matching \'{$a}\'';
 $string['filter:manage'] = 'Manage local filter settings';
 $string['frontpageuser'] = 'Authenticated user on frontpage';
-$string['frontpageuserdescription'] = 'All logged in users in the frontpage course.';
+$string['frontpageuserdescription'] = 'All logged-in users in the frontpage course.';
 $string['globalrole'] = 'System role';
-$string['globalroleswarning'] = 'WARNING! Any roles you assign from this page will apply to the assigned users throughout the entire system, including the front page and all the courses.';
+$string['globalroleswarning'] = 'Warning: Any roles you assign from this page will apply to users throughout the entire system, including the front page and all courses.';
 $string['gotoassignroles'] = 'Go to Assign roles for this {$a->contextlevel}';
 $string['gotoassignsystemroles'] = 'Go to Assign system roles';
 $string['grade:edit'] = 'Edit grades';
@@ -328,6 +333,8 @@ $string['overrideroles'] = 'Override roles';
 $string['overriderolesin'] = 'Override roles in {$a}';
 $string['overrides'] = 'Overrides';
 $string['overridesbycontext'] = 'Overrides (by context)';
+$string['payment:manageaccounts'] = 'Manage accounts';
+$string['payment:viewpayments'] = 'View payments';
 $string['permission'] = 'Permission';
 $string['permission_help'] = 'Permissions are capability settings. There are 4 options:
 
@@ -396,7 +403,7 @@ $string['role:safeoverride'] = 'Override safe permissions for others';
 $string['roleselect'] = 'Select role';
 $string['rolesforuser'] = 'Roles for user {$a}';
 $string['roleshortname'] = 'Short name';
-$string['roleshortname_help'] = 'Role short name is a low level role identifier in which only ASCII alphanumeric characters are allowed. Do not change short names of standard roles.';
+$string['roleshortname_help'] = 'Role short name is a low level role identifier in which only ASCII alphanumeric characters are allowed.';
 $string['role:switchroles'] = 'Switch to other roles';
 $string['roletoassign'] = 'Role to assign';
 $string['roletooverride'] = 'Role to override';
@@ -428,6 +435,7 @@ $string['site:mnetloginfromremote'] = 'Login from a remote application via MNet'
 $string['site:mnetlogintoremote'] = 'Roam to a remote application via MNet';
 $string['site:readallmessages'] = 'Read all messages on site';
 $string['site:restore'] = 'Restore courses';
+$string['site:senderrormessage'] = 'Send a message to the support user from the error page';
 $string['site:sendmessage'] = 'Send messages to any user';
 $string['site:trustcontent'] = 'Trust submitted content';
 $string['site:uploadusers'] = 'Upload new users from file';
