@@ -37,7 +37,7 @@ $string['filtertype'] = 'Filter type';
 $string['match'] = 'Match';
 $string['matchofthefollowing'] = 'of the following:';
 $string['moodlenetprofile'] = 'MoodleNet profile ID';
-$string['moodlenetprofile_help'] = 'This field represents your MoodleNet profile ID which is used to link your MoodleNet profile to Moodle. It expects a WebFinger-compliant URI.';
+$string['moodlenetprofile_help'] = 'Your MoodleNet profile ID links your MoodleNet profile with this site.';
 $string['placeholdertypeorselect'] = 'Type or select...';
 $string['placeholdertype'] = 'Type...';
 $string['privacy:courserequestpath'] = 'Requested courses';
