@@ -124,10 +124,9 @@ function behat_get_error_string($errtype) {
  * @param string $errstr
  * @param string $errfile
  * @param int $errline
- * @param array $errcontext
  * @return bool
  */
-function behat_error_handler($errno, $errstr, $errfile, $errline, $errcontext) {
+function behat_error_handler($errno, $errstr, $errfile, $errline) {
 
     // If is preceded by an @ we don't show it.
     if (!error_reporting()) {
@@ -149,7 +148,7 @@ function behat_error_handler($errno, $errstr, $errfile, $errline, $errcontext) {
     }
 
     // Using the default one in case there is a fatal catchable error.
-    default_error_handler($errno, $errstr, $errfile, $errline, $errcontext);
+    default_error_handler($errno, $errstr, $errfile, $errline);
 
     $errnostr = behat_get_error_string($errno);
 
@@ -549,3 +548,24 @@ function cli_execute_parallel($cmds, $cwd = null, $delay = 0) {
     }
     return $processes;
 }
+
+/**
+ * Get command flags for an option/value combination
+ *
+ * @param string $option
+ * @param string|bool|null $value
+ * @return string
+ */
+function behat_get_command_flags(string $option, $value): string {
+    $commandoptions = '';
+    if (is_bool($value)) {
+        if ($value) {
+            return " --{$option}";
+        } else {
+            return " --no-{$option}";
+        }
+    } else if ($value !== null) {
+        return " --$option=\"$value\"";
+    }
+    return '';
+}
