@@ -32,7 +32,7 @@
  */
 
 var CSS = {
-        RESPONSIVE: 'img-responsive',
+        RESPONSIVE: 'img-fluid',
         INPUTALIGNMENT: 'atto_image_alignment',
         INPUTALT: 'atto_image_altentry',
         INPUTHEIGHT: 'atto_image_heightentry',
@@ -41,6 +41,7 @@ var CSS = {
         INPUTSIZE: 'atto_image_size',
         INPUTWIDTH: 'atto_image_widthentry',
         IMAGEALTWARNING: 'atto_image_altwarning',
+        IMAGEURLWARNING: 'atto_image_urlwarning',
         IMAGEBROWSER: 'openimagebrowser',
         IMAGEPRESENTATION: 'atto_image_presentation',
         INPUTCONSTRAIN: 'atto_image_constrain',
@@ -49,6 +50,10 @@ var CSS = {
         IMAGEPREVIEWBOX: 'atto_image_preview_box',
         ALIGNSETTINGS: 'atto_image_button'
     },
+    FORMNAMES = {
+        URL: 'urlentry',
+        ALT: 'altentry'
+    },
     SELECTORS = {
         INPUTURL: '.' + CSS.INPUTURL
     },
@@ -85,7 +90,10 @@ var CSS = {
             margin: '0 0 0 0.5em'
         }
     ],
-
+    DEFAULTS = {
+        WIDTH: 160,
+        HEIGHT: 160,
+    },
     REGEX = {
         ISPERCENT: /\d+%/
     },
@@ -94,13 +102,17 @@ var CSS = {
 
     TEMPLATE = '' +
             '<form class="atto_form">' +
-
                 // Add the repository browser button.
+                '<div style="display:none" role="alert" class="alert alert-warning mb-1 {{CSS.IMAGEURLWARNING}}">' +
+                    '<label for="{{elementid}}_{{CSS.INPUTURL}}">' +
+                    '{{get_string "imageurlrequired" component}}' +
+                    '</label>' +
+                '</div>' +
                 '{{#if showFilepicker}}' +
                     '<div class="mb-1">' +
                         '<label for="{{elementid}}_{{CSS.INPUTURL}}">{{get_string "enterurl" component}}</label>' +
                         '<div class="input-group input-append w-100">' +
-                            '<input class="form-control {{CSS.INPUTURL}}" type="url" ' +
+                            '<input name="{{FORMNAMES.URL}}" class="form-control {{CSS.INPUTURL}}" type="url" ' +
                             'id="{{elementid}}_{{CSS.INPUTURL}}" size="32"/>' +
                             '<span class="input-group-append">' +
                                 '<button class="btn btn-secondary {{CSS.IMAGEBROWSER}}" type="button">' +
@@ -111,19 +123,21 @@ var CSS = {
                 '{{else}}' +
                     '<div class="mb-1">' +
                         '<label for="{{elementid}}_{{CSS.INPUTURL}}">{{get_string "enterurl" component}}</label>' +
-                        '<input class="form-control fullwidth {{CSS.INPUTURL}}" type="url" ' +
+                        '<input name="{{FORMNAMES.URL}}" class="form-control fullwidth {{CSS.INPUTURL}}" type="url" ' +
                         'id="{{elementid}}_{{CSS.INPUTURL}}" size="32"/>' +
                     '</div>' +
                 '{{/if}}' +
 
-                // Add the Alt box.
                 '<div style="display:none" role="alert" class="alert alert-warning mb-1 {{CSS.IMAGEALTWARNING}}">' +
+                    '<label for="{{elementid}}_{{CSS.INPUTALT}}">' +
                     '{{get_string "presentationoraltrequired" component}}' +
+                    '</label>' +
                 '</div>' +
+                // Add the Alt box.
                 '<div class="mb-1">' +
                 '<label for="{{elementid}}_{{CSS.INPUTALT}}">{{get_string "enteralt" component}}</label>' +
                 '<textarea class="form-control fullwidth {{CSS.INPUTALT}}" ' +
-                'id="{{elementid}}_{{CSS.INPUTALT}}" maxlength="125"></textarea>' +
+                'id="{{elementid}}_{{CSS.INPUTALT}}" name="{{FORMNAMES.ALT}}" maxlength="125"></textarea>' +
 
                 // Add the character count.
                 '<div id="the-count" class="d-flex justify-content-end small">' +
@@ -250,7 +264,7 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         this.editor.on('paste', this._handlePaste, this);
         this.editor.on('drop', this._handleDragDrop, this);
 
-        // e.preventDefault needed to stop the default event from clobbering the desired behaviour in some browsers.
+        // ...e.preventDefault needed to stop the default event from clobbering the desired behaviour in some browsers.
         this.editor.on('dragover', function(e) {
             e.preventDefault();
         }, this);
@@ -264,16 +278,15 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
      *
      * @method _handleDragDrop
      * @param {EventFacade} e
-     * @return {boolean} false if we handled the event, else true.
      * @private
      */
     _handleDragDrop: function(e) {
         if (!e._event || !e._event.dataTransfer) {
             // Drop not fully supported in this browser.
-            return true;
+            return;
         }
 
-        return this._handlePasteOrDropHelper(e, e._event.dataTransfer);
+        this._handlePasteOrDropHelper(e, e._event.dataTransfer);
     },
 
     /**
@@ -319,13 +332,7 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         }
 
         if (didUpload) {
-            // We handled this.
             e.preventDefault();
-            e.stopPropagation();
-            return false;
-        } else {
-            // Let someone else try to handle it.
-            return true;
         }
     },
 
@@ -356,6 +363,11 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
 
         host.saveSelection();
 
+        // Trigger form upload start events.
+        require(['core_form/events'], function(FormEvent) {
+            FormEvent.triggerUploadStarted(self.editor.get('id'));
+        });
+
         var options = host.get('filepickeroptions').image,
             savepath = (options.savepath === undefined) ? '/' : options.savepath,
             formData = new FormData(),
@@ -410,6 +422,10 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
                             if (placeholder) {
                                 placeholder.remove(true);
                             }
+                            // Trigger form upload complete events.
+                            require(['core_form/events'], function(FormEvent) {
+                                FormEvent.triggerUploadCompleted(self.editor.get('id'));
+                            });
                             throw new M.core.ajaxException(result);
                         }
 
@@ -424,7 +440,8 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
                         // Replace placeholder with actual image.
                         newhtml = template({
                             url: file.url,
-                            presentation: true
+                            presentation: true,
+                            classlist: CSS.RESPONSIVE
                         });
                         newimage = Y.Node.create(newhtml);
                         if (placeholder) {
@@ -436,12 +453,20 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
                     }
                 } else {
                     Y.use('moodle-core-notification-alert', function() {
+                        // Trigger form upload complete events.
+                        require(['core_form/events'], function(FormEvent) {
+                            FormEvent.triggerUploadCompleted(self.editor.get('id'));
+                        });
                         new M.core.alert({message: M.util.get_string('servererror', 'moodle')});
                     });
                     if (placeholder) {
                         placeholder.remove(true);
                     }
                 }
+                // Trigger form upload complete events.
+                require(['core_form/events'], function(FormEvent) {
+                    FormEvent.triggerUploadCompleted(self.editor.get('id'));
+                });
             }
         };
         xhr.open("POST", M.cfg.wwwroot + '/repository/repository_ajax.php?action=upload', true);
@@ -519,22 +544,23 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         image.onload = function() {
             var input, currentwidth, currentheight, widthRatio, heightRatio;
 
+            // Store dimensions of the raw image, falling back to defaults for images without dimensions (e.g. SVG).
             self._rawImageDimensions = {
-                width: this.width,
-                height: this.height
+                width: this.width || DEFAULTS.WIDTH,
+                height: this.height || DEFAULTS.HEIGHT,
             };
 
             input = self._form.one('.' + CSS.INPUTWIDTH);
             currentwidth = input.get('value');
             if (currentwidth === '') {
-                input.set('value', this.width);
-                currentwidth = "" + this.width;
+                input.set('value', self._rawImageDimensions.width);
+                currentwidth = "" + self._rawImageDimensions.width;
             }
             input = self._form.one('.' + CSS.INPUTHEIGHT);
             currentheight = input.get('value');
             if (currentheight === '') {
-                input.set('value', this.height);
-                currentheight = "" + this.height;
+                input.set('value', self._rawImageDimensions.height);
+                currentheight = "" + self._rawImageDimensions.height;
             }
             input = self._form.one('.' + CSS.IMAGEPREVIEW);
             input.setAttribute('src', this.src);
@@ -545,13 +571,10 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
             input = self._form.one('.' + CSS.INPUTCONSTRAIN);
             if (currentwidth.match(REGEX.ISPERCENT) && currentheight.match(REGEX.ISPERCENT)) {
                 input.set('checked', currentwidth === currentheight);
+            } else if (this.width === 0 || this.height === 0) {
+                // If we don't have both dimensions of the image, we can't auto-size it, so disable control.
+                input.set('disabled', 'disabled');
             } else {
-                if (this.width === 0) {
-                    this.width = 1;
-                }
-                if (this.height === 0) {
-                    this.height = 1;
-                }
                 // This is the same as comparing to 3 decimal places.
                 widthRatio = Math.round(1000 * parseInt(currentwidth, 10) / this.width);
                 heightRatio = Math.round(1000 * parseInt(currentheight, 10) / this.height);
@@ -582,6 +605,7 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
             content = Y.Node.create(template({
                 elementid: this.get('host').get('elementid'),
                 CSS: CSS,
+                FORMNAMES: FORMNAMES,
                 component: COMPONENTNAME,
                 showFilepicker: canShowFilepicker,
                 alignments: ALIGNMENTS
@@ -593,8 +617,9 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         this._applyImageProperties(this._form);
 
         this._form.one('.' + CSS.INPUTURL).on('blur', this._urlChanged, this);
-        this._form.one('.' + CSS.IMAGEPRESENTATION).on('change', this._updateWarning, this);
-        this._form.one('.' + CSS.INPUTALT).on('change', this._updateWarning, this);
+        this._form.one('.' + CSS.INPUTURL).on('change', this._hasErrorUrlField, this);
+        this._form.one('.' + CSS.IMAGEPRESENTATION).on('change', this._hasErrorAltField, this);
+        this._form.one('.' + CSS.INPUTALT).on('blur', this._hasErrorAltField, this);
         this._form.one('.' + CSS.INPUTWIDTH).on('blur', this._autoAdjustSize, this);
         this._form.one('.' + CSS.INPUTHEIGHT).on('blur', this._autoAdjustSize, this, true);
         this._form.one('.' + CSS.INPUTCONSTRAIN).on('change', function(event) {
@@ -602,7 +627,6 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
                 this._autoAdjustSize(event);
             }
         }, this);
-        this._form.one('.' + CSS.INPUTURL).on('blur', this._urlChanged, this);
         this._form.one('.' + CSS.INPUTSUBMIT).on('click', this._setImage, this);
 
         if (canShowFilepicker) {
@@ -1029,6 +1053,38 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         return CSS.ALIGNSETTINGS + '_' + alignment;
     },
 
+    _toggleVisibility: function(selector, predicate) {
+        var form = this._form;
+        var element = form.all(selector);
+        element.setStyle('display', predicate ? 'block' : 'none');
+    },
+
+    _toggleAriaInvalid: function(selectors, predicate) {
+        var form = this._form;
+        selectors.forEach(function(selector) {
+            var element = form.all(selector);
+            element.setAttribute('aria-invalid', predicate);
+        });
+    },
+
+    _hasErrorUrlField: function() {
+        var form = this._form;
+        var url = form.one('.' + CSS.INPUTURL).get('value');
+        var urlerror = url === '';
+        this._toggleVisibility('.' + CSS.IMAGEURLWARNING, urlerror);
+        this._toggleAriaInvalid(['.' + CSS.INPUTURL], urlerror);
+        return urlerror;
+    },
+
+    _hasErrorAltField: function() {
+        var form = this._form;
+        var alt = form.one('.' + CSS.INPUTALT).get('value');
+        var presentation = form.one('.' + CSS.IMAGEPRESENTATION).get('checked');
+        var imagealterror = alt === '' && !presentation;
+        this._toggleVisibility('.' + CSS.IMAGEALTWARNING, imagealterror);
+        this._toggleAriaInvalid(['.' + CSS.INPUTALT, '.' + CSS.IMAGEPRESENTATION], imagealterror);
+        return imagealterror;
+    },
     /**
      * Update the alt text warning live.
      *
@@ -1037,23 +1093,11 @@ Y.namespace('M.atto_image').Button = Y.Base.create('button', Y.M.editor_atto.Edi
      * @private
      */
     _updateWarning: function() {
-        var form = this._form,
-            state = true,
-            alt = form.one('.' + CSS.INPUTALT).get('value'),
-            presentation = form.one('.' + CSS.IMAGEPRESENTATION).get('checked');
-        if (alt === '' && !presentation) {
-            form.one('.' + CSS.IMAGEALTWARNING).setStyle('display', 'block');
-            form.one('.' + CSS.INPUTALT).setAttribute('aria-invalid', true);
-            form.one('.' + CSS.IMAGEPRESENTATION).setAttribute('aria-invalid', true);
-            state = true;
-        } else {
-            form.one('.' + CSS.IMAGEALTWARNING).setStyle('display', 'none');
-            form.one('.' + CSS.INPUTALT).setAttribute('aria-invalid', false);
-            form.one('.' + CSS.IMAGEPRESENTATION).setAttribute('aria-invalid', false);
-            state = false;
-        }
+        var urlerror = this._hasErrorUrlField();
+        var imagealterror = this._hasErrorAltField();
+        var haserrors = urlerror || imagealterror;
         this.getDialogue().centerDialogue();
-        return state;
+        return haserrors;
     },
 
     /**
