@@ -63,7 +63,7 @@ abstract class advanced_testcase extends base_testcase {
      * Runs the bare test sequence.
      * @return void
      */
-    final public function runBare() {
+    final public function runBare(): void {
         global $DB;
 
         if (phpunit_util::$lastdbwrites != $DB->perf_get_writes()) {
@@ -132,6 +132,9 @@ abstract class advanced_testcase extends base_testcase {
             self::resetAllData(true);
         }
 
+        // Reset context cache.
+        context_helper::reset_caches();
+
         // make sure test did not forget to close transaction
         if ($DB->is_transaction_started()) {
             self::resetAllData();
@@ -143,88 +146,119 @@ abstract class advanced_testcase extends base_testcase {
         }
     }
 
-    /**
-     * Creates a new FlatXmlDataSet with the given $xmlFile. (absolute path.)
-     *
-     * @param string $xmlFile
-     * @return PHPUnit\DbUnit\DataSet\FlatXmlDataSet
-     */
-    protected function createFlatXMLDataSet($xmlFile) {
-        return new PHPUnit\DbUnit\DataSet\FlatXmlDataSet($xmlFile);
-    }
-
     /**
      * Creates a new XMLDataSet with the given $xmlFile. (absolute path.)
      *
+     * @deprecated since Moodle 3.10 - See MDL-67673 and MDL-64600 for more info.
+     * @todo This will be removed for Moodle 4.2 as part of MDL-69882.
+     *
      * @param string $xmlFile
-     * @return PHPUnit\DbUnit\DataSet\XmlDataSet
+     * @return phpunit_dataset
      */
     protected function createXMLDataSet($xmlFile) {
-        return new PHPUnit\DbUnit\DataSet\XmlDataSet($xmlFile);
+        debugging(__FUNCTION__ . '() is deprecated. Please use dataset_from_files() instead.', DEBUG_DEVELOPER);
+        return $this->dataset_from_files([$xmlFile]);
     }
 
     /**
      * Creates a new CsvDataSet from the given array of csv files. (absolute paths.)
      *
+     * @deprecated since Moodle 3.10 - See MDL-67673 and MDL-64600 for more info.
+     * @todo This will be removed for Moodle 4.2 as part of MDL-69882.
+     *
      * @param array $files array tablename=>cvsfile
-     * @param string $delimiter
-     * @param string $enclosure
-     * @param string $escape
-     * @return PHPUnit\DbUnit\DataSet\CsvDataSet
+     * @param string $delimiter unused
+     * @param string $enclosure unused
+     * @param string $escape unused
+     * @return phpunit_dataset
      */
     protected function createCsvDataSet($files, $delimiter = ',', $enclosure = '"', $escape = '"') {
-        $dataSet = new PHPUnit\DbUnit\DataSet\CsvDataSet($delimiter, $enclosure, $escape);
-        foreach($files as $table=>$file) {
-            $dataSet->addTable($table, $file);
-        }
-        return $dataSet;
+        debugging(__FUNCTION__ . '() is deprecated. Please use dataset_from_files() instead.', DEBUG_DEVELOPER);
+        return $this->dataset_from_files($files);
     }
 
     /**
      * Creates new ArrayDataSet from given array
      *
+     * @deprecated since Moodle 3.10 - See MDL-67673 and MDL-64600 for more info.
+     * @todo This will be removed for Moodle 4.2 as part of MDL-69882.
+     *
      * @param array $data array of tables, first row in each table is columns
-     * @return phpunit_ArrayDataSet
+     * @return phpunit_dataset
      */
     protected function createArrayDataSet(array $data) {
-        return new phpunit_ArrayDataSet($data);
+        debugging(__FUNCTION__ . '() is deprecated. Please use dataset_from_array() instead.', DEBUG_DEVELOPER);
+        return $this->dataset_from_array($data);
     }
 
     /**
      * Load date into moodle database tables from standard PHPUnit data set.
      *
+     * @deprecated since Moodle 3.10 - See MDL-67673 and MDL-64600 for more info.
+     * @todo This will be removed for Moodle 4.2 as part of MDL-69882.
+     *
      * Note: it is usually better to use data generators
      *
-     * @param PHPUnit\DbUnit\DataSet\IDataSet $dataset
+     * @param phpunit_dataset $dataset
      * @return void
      */
-    protected function loadDataSet(PHPUnit\DbUnit\DataSet\IDataSet $dataset) {
-        global $DB;
+    protected function loadDataSet(phpunit_dataset $dataset) {
+        debugging(__FUNCTION__ . '() is deprecated. Please use dataset->to_database() instead.', DEBUG_DEVELOPER);
+        $dataset->to_database();
+    }
 
-        $structure = phpunit_util::get_tablestructure();
+    /**
+     * Creates a new dataset from CVS/XML files.
+     *
+     * This method accepts an array of full paths to CSV or XML files to be loaded
+     * into the dataset. For CSV files, the name of the table which the file belongs
+     * to needs to be specified. Example:
+     *
+     *   $fullpaths = [
+     *       '/path/to/users.xml',
+     *       'course' => '/path/to/courses.csv',
+     *   ];
+     *
+     * @since Moodle 3.10
+     *
+     * @param array $files full paths to CSV or XML files to load.
+     * @return phpunit_dataset
+     */
+    protected function dataset_from_files(array $files) {
+        // We ignore $delimiter, $enclosure and $escape, use the default ones in your fixtures.
+        $dataset = new phpunit_dataset();
+        $dataset->from_files($files);
+        return $dataset;
+    }
 
-        foreach($dataset->getTableNames() as $tablename) {
-            $table = $dataset->getTable($tablename);
-            $metadata = $dataset->getTableMetaData($tablename);
-            $columns = $metadata->getColumns();
+    /**
+     * Creates a new dataset from string (CSV or XML).
+     *
+     * @since Moodle 3.10
+     *
+     * @param string $content contents (CSV or XML) to load.
+     * @param string $type format of the content to be loaded (csv or xml).
+     * @param string $table name of the table which the file belongs to (only for CSV files).
+     * @return phpunit_dataset
+     */
+    protected function dataset_from_string(string $content, string $type, ?string $table = null) {
+        $dataset = new phpunit_dataset();
+        $dataset->from_string($content, $type, $table);
+        return $dataset;
+    }
 
-            $doimport = false;
-            if (isset($structure[$tablename]['id']) and $structure[$tablename]['id']->auto_increment) {
-                $doimport = in_array('id', $columns);
-            }
-
-            for($r=0; $r<$table->getRowCount(); $r++) {
-                $record = $table->getRow($r);
-                if ($doimport) {
-                    $DB->import_record($tablename, $record);
-                } else {
-                    $DB->insert_record($tablename, $record);
-                }
-            }
-            if ($doimport) {
-                $DB->get_manager()->reset_sequence(new xmldb_table($tablename));
-            }
-        }
+    /**
+     * Creates a new dataset from PHP array.
+     *
+     * @since Moodle 3.10
+     *
+     * @param array $data array of tables, see {@see phpunit_dataset::from_array()} for supported formats.
+     * @return phpunit_dataset
+     */
+    protected function dataset_from_array(array $data) {
+        $dataset = new phpunit_dataset();
+        $dataset->from_array($data);
+        return $dataset;
     }
 
     /**
@@ -699,7 +733,7 @@ abstract class advanced_testcase extends base_testcase {
             }
 
             cron_prepare_core_renderer();
-            $this->setUser($user);
+            cron_setup_user($user);
 
             $task->execute();
             \core\task\manager::adhoc_task_complete($task);
