@@ -604,8 +604,8 @@ class behat_navigation extends behat_base {
         $dividercount = substr_count($page, ' > ');
         if ($dividercount === 0) {
             return ['core', $page];
-        } else if ($dividercount === 1) {
-            list($component, $name) = explode(' > ', $page);
+        } else if ($dividercount >= 1) {
+            [$component, $name] = explode(' > ', $page, 2);
             if ($component === 'core') {
                 throw new coding_exception('Do not specify the component "core > ..." for core pages.');
             }
@@ -718,6 +718,7 @@ class behat_navigation extends behat_base {
      * | Page type                  | Identifier meaning        | description                          |
      * | Category                   | category idnumber         | List of courses in that category.    |
      * | Course                     | course shortname          | Main course home pag                 |
+     * | Course editing             | course shortname          | Edit settings page for the course    |
      * | Activity                   | activity idnumber         | Start page for that activity         |
      * | Activity editing           | activity idnumber         | Edit settings page for that activity |
      * | [modname] Activity         | activity name or idnumber | Start page for that activity         |
@@ -733,8 +734,6 @@ class behat_navigation extends behat_base {
      * @throws Exception with a meaningful error message if the specified page cannot be found.
      */
     protected function resolve_core_page_instance_url(string $type, string $identifier): moodle_url {
-        global $DB;
-
         $type = strtolower($type);
 
         switch ($type) {
@@ -745,6 +744,14 @@ class behat_navigation extends behat_base {
                 }
                 return new moodle_url('/course/index.php', ['categoryid' => $categoryid]);
 
+            case 'course editing':
+                $courseid = $this->get_course_id($identifier);
+                if (!$courseid) {
+                    throw new Exception('The specified course with shortname, fullname, or idnumber "' .
+                        $identifier . '" does not exist');
+                }
+                return new moodle_url('/course/edit.php', ['id' => $courseid]);
+
             case 'course':
                 $courseid = $this->get_course_id($identifier);
                 if (!$courseid) {
@@ -810,9 +817,8 @@ class behat_navigation extends behat_base {
      * @return void
      */
     public function i_am_on_course_homepage($coursefullname) {
-        global $DB;
-        $course = $DB->get_record("course", array("fullname" => $coursefullname), 'id', MUST_EXIST);
-        $url = new moodle_url('/course/view.php', ['id' => $course->id]);
+        $courseid = $this->get_course_id($coursefullname);
+        $url = new moodle_url('/course/view.php', ['id' => $courseid]);
         $this->execute('behat_general::i_visit', [$url]);
     }
 
@@ -825,10 +831,8 @@ class behat_navigation extends behat_base {
      * @return void
      */
     public function i_am_on_course_homepage_with_editing_mode_on($coursefullname) {
-        global $DB;
-
-        $course = $DB->get_record("course", array("fullname" => $coursefullname), 'id', MUST_EXIST);
-        $url = new moodle_url('/course/view.php', ['id' => $course->id]);
+        $courseid = $this->get_course_id($coursefullname);
+        $url = new moodle_url('/course/view.php', ['id' => $courseid]);
 
         if ($this->running_javascript() && $sesskey = $this->get_sesskey()) {
             // Javascript is running so it is possible to grab the session ket and jump straight to editing mode.
