@@ -14,23 +14,17 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
+namespace core;
+
 /**
- * Tests groups subsystems.
+ * Unit tests for lib/grouplib.php
  *
- * @package    core_group
- * @category   phpunit
+ * @package    core
  * @copyright  2007 onwards Martin Dougiamas (http://dougiamas.com)
  * @author     Andrew Nicols
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-
-defined('MOODLE_INTERNAL') || die();
-
-/**
- * Unit tests for lib/grouplib.php
- * @group core_group
- */
-class core_grouplib_testcase extends advanced_testcase {
+class grouplib_test extends \advanced_testcase {
 
     public function test_groups_get_group_by_idnumber() {
         $this->resetAfterTest(true);
@@ -184,7 +178,7 @@ class core_grouplib_testcase extends advanced_testcase {
         $generator = $this->getDataGenerator();
 
         $course = $generator->create_course();
-        $coursecontext = context_course::instance($course->id);
+        $coursecontext = \context_course::instance($course->id);
         $student1 = $generator->create_user();
         $student2 = $generator->create_user();
         $plugin = enrol_get_plugin('manual');
@@ -213,6 +207,161 @@ class core_grouplib_testcase extends advanced_testcase {
         $this->assertTrue(array_key_exists($student1->id, $users));
     }
 
+    public function test_groups_get_members_ids_sql_multiple_groups() {
+        global $DB;
+
+        $this->resetAfterTest(true);
+
+        $generator = $this->getDataGenerator();
+
+        $course = $generator->create_course();
+        $student1 = $generator->create_user();
+        $student2 = $generator->create_user();
+        $plugin = enrol_get_plugin('manual');
+        $role = $DB->get_record('role', array('shortname' => 'student'));
+        $group1 = $generator->create_group(array('courseid' => $course->id));
+        $group2 = $generator->create_group(array('courseid' => $course->id));
+        $groupids = [
+            $group1->id,
+            $group2->id,
+        ];
+        $instance = $DB->get_record('enrol', array(
+                'courseid' => $course->id,
+                'enrol' => 'manual',
+        ));
+
+        $this->assertNotEquals($instance, false);
+
+        // Enrol users in the course.
+        $plugin->enrol_user($instance, $student1->id, $role->id);
+        $plugin->enrol_user($instance, $student2->id, $role->id);
+
+        list($sql, $params) = groups_get_members_ids_sql($groupids);
+
+        // Test an empty group.
+        $users = $DB->get_records_sql($sql, $params);
+        $this->assertFalse(array_key_exists($student1->id, $users));
+
+        // Test with a member of one of the two group.
+        groups_add_member($group1->id, $student1->id);
+        $users = $DB->get_records_sql($sql, $params);
+        $this->assertTrue(array_key_exists($student1->id, $users));
+
+        // Test with members of two groups.
+        groups_add_member($group2->id, $student2->id);
+        $users = $DB->get_records_sql($sql, $params);
+        $this->assertTrue(array_key_exists($student1->id, $users));
+        $this->assertTrue(array_key_exists($student2->id, $users));
+    }
+
+    public function test_groups_get_members_ids_sql_multiple_groups_join_types() {
+        global $DB;
+
+        $this->resetAfterTest(true);
+
+        $generator = $this->getDataGenerator();
+
+        $course = $generator->create_course();
+        $student1 = $generator->create_user();
+        $student2 = $generator->create_user();
+        $student3 = $generator->create_user();
+        $student4 = $generator->create_user();
+        $student5 = $generator->create_user();
+        $student6 = $generator->create_user();
+        $plugin = enrol_get_plugin('manual');
+        $role = $DB->get_record('role', array('shortname' => 'student'));
+        $group1 = $generator->create_group(array('courseid' => $course->id));
+        $group2 = $generator->create_group(array('courseid' => $course->id));
+        $group3 = $generator->create_group(array('courseid' => $course->id));
+        // Only groups 1 and 2 specified in SQL (group 3 helps cover the None case).
+        $groupids = [
+            $group1->id,
+            $group2->id,
+        ];
+        $instance = $DB->get_record('enrol', array(
+                'courseid' => $course->id,
+                'enrol' => 'manual',
+        ));
+
+        $this->assertNotEquals($instance, false);
+
+        // Enrol users in the course.
+        $plugin->enrol_user($instance, $student1->id, $role->id);
+        $plugin->enrol_user($instance, $student2->id, $role->id);
+        $plugin->enrol_user($instance, $student3->id, $role->id);
+        $plugin->enrol_user($instance, $student4->id, $role->id);
+        $plugin->enrol_user($instance, $student5->id, $role->id);
+        $plugin->enrol_user($instance, $student6->id, $role->id);
+
+        // Generate SQL with the different groups join types for members of group1 and group2.
+        list($sqlany, $paramsany) = groups_get_members_ids_sql($groupids, null, GROUPS_JOIN_ANY);
+        list($sqlall, $paramsall) = groups_get_members_ids_sql($groupids, null, GROUPS_JOIN_ALL);
+        list($sqlnone, $paramsnone) = groups_get_members_ids_sql($groupids, null, GROUPS_JOIN_NONE);
+
+        // Any - Test empty groups, no matches.
+        $users = $DB->get_records_sql($sqlany, $paramsany);
+        $this->assertFalse(array_key_exists($student1->id, $users));
+        $this->assertFalse(array_key_exists($student2->id, $users));
+        $this->assertFalse(array_key_exists($student3->id, $users));
+        $this->assertFalse(array_key_exists($student4->id, $users));
+        $this->assertFalse(array_key_exists($student5->id, $users));
+        $this->assertFalse(array_key_exists($student6->id, $users));
+
+        // All - Test empty groups, no matches.
+        $users = $DB->get_records_sql($sqlall, $paramsall);
+        $this->assertFalse(array_key_exists($student1->id, $users));
+        $this->assertFalse(array_key_exists($student2->id, $users));
+        $this->assertFalse(array_key_exists($student3->id, $users));
+        $this->assertFalse(array_key_exists($student4->id, $users));
+        $this->assertFalse(array_key_exists($student5->id, $users));
+        $this->assertFalse(array_key_exists($student6->id, $users));
+
+        // None - Test empty groups, all match.
+        $users = $DB->get_records_sql($sqlnone, $paramsnone);
+        $this->assertTrue(array_key_exists($student1->id, $users));
+        $this->assertTrue(array_key_exists($student2->id, $users));
+        $this->assertTrue(array_key_exists($student3->id, $users));
+        $this->assertTrue(array_key_exists($student4->id, $users));
+        $this->assertTrue(array_key_exists($student5->id, $users));
+        $this->assertTrue(array_key_exists($student6->id, $users));
+
+        // Assign various group member combinations.
+        groups_add_member($group1->id, $student1->id);
+        groups_add_member($group1->id, $student2->id);
+        groups_add_member($group1->id, $student3->id);
+        groups_add_member($group2->id, $student2->id);
+        groups_add_member($group2->id, $student3->id);
+        groups_add_member($group2->id, $student4->id);
+        groups_add_member($group3->id, $student5->id);
+
+        // Any - Test students in one or both of groups 1 and 2 matched.
+        $users = $DB->get_records_sql($sqlany, $paramsany);
+        $this->assertTrue(array_key_exists($student1->id, $users));
+        $this->assertTrue(array_key_exists($student2->id, $users));
+        $this->assertTrue(array_key_exists($student3->id, $users));
+        $this->assertTrue(array_key_exists($student4->id, $users));
+        $this->assertFalse(array_key_exists($student5->id, $users));
+        $this->assertFalse(array_key_exists($student6->id, $users));
+
+        // All - Test only students in both groups 1 and 2 matched.
+        $users = $DB->get_records_sql($sqlall, $paramsall);
+        $this->assertTrue(array_key_exists($student2->id, $users));
+        $this->assertTrue(array_key_exists($student3->id, $users));
+        $this->assertFalse(array_key_exists($student1->id, $users));
+        $this->assertFalse(array_key_exists($student4->id, $users));
+        $this->assertFalse(array_key_exists($student5->id, $users));
+        $this->assertFalse(array_key_exists($student6->id, $users));
+
+        // None - Test only students not in group 1 or 2 matched.
+        $users = $DB->get_records_sql($sqlnone, $paramsnone);
+        $this->assertTrue(array_key_exists($student5->id, $users));
+        $this->assertTrue(array_key_exists($student6->id, $users));
+        $this->assertFalse(array_key_exists($student1->id, $users));
+        $this->assertFalse(array_key_exists($student2->id, $users));
+        $this->assertFalse(array_key_exists($student3->id, $users));
+        $this->assertFalse(array_key_exists($student4->id, $users));
+    }
+
     public function test_groups_get_members_ids_sql_valid_context() {
         global $DB;
 
@@ -221,7 +370,7 @@ class core_grouplib_testcase extends advanced_testcase {
         $generator = $this->getDataGenerator();
 
         $course = $generator->create_course();
-        $coursecontext = context_course::instance($course->id);
+        $coursecontext = \context_course::instance($course->id);
         $student1 = $generator->create_user();
         $student2 = $generator->create_user();
         $plugin = enrol_get_plugin('manual');
@@ -256,7 +405,7 @@ class core_grouplib_testcase extends advanced_testcase {
         $generator = $this->getDataGenerator();
 
         $course = $generator->create_course();
-        $coursecontext = context_course::instance($course->id);
+        $coursecontext = \context_course::instance($course->id);
         $student1 = $generator->create_user();
         $student2 = $generator->create_user();
         $plugin = enrol_get_plugin('manual');
@@ -289,7 +438,7 @@ class core_grouplib_testcase extends advanced_testcase {
         $generator = $this->getDataGenerator();
 
         $course = $generator->create_course();
-        $coursecontext = context_course::instance($course->id);
+        $coursecontext = \context_course::instance($course->id);
         $student1 = $generator->create_user();
         $student2 = $generator->create_user();
         $plugin = enrol_get_plugin('manual');
@@ -310,7 +459,7 @@ class core_grouplib_testcase extends advanced_testcase {
         groups_add_member($group->id, $student1->id);
 
         // Test with members at any group and with an invalid $context.
-        $syscontext = context_system::instance();
+        $syscontext = \context_system::instance();
         $this->expectException('coding_exception');
         list($sql, $params) = groups_get_members_ids_sql(USERSWITHOUTGROUP, $syscontext);
     }
@@ -486,8 +635,8 @@ class core_grouplib_testcase extends advanced_testcase {
         $this->assertSame($group3->name, $data->groups[$group3->id]->name);
 
         // Check we have the expected number of groupings.
-        $this->assertContains($grouping1->id, array_keys($data->groupings));
-        $this->assertContains($grouping2->id, array_keys($data->groupings));
+        $this->assertArrayHasKey($grouping1->id, $data->groupings);
+        $this->assertArrayHasKey($grouping2->id, $data->groupings);
 
         // Test a grouping-id is mapped correctly.
         $this->assertEquals($grouping2->name, $data->groupings[$grouping2->id]->name);
@@ -502,10 +651,10 @@ class core_grouplib_testcase extends advanced_testcase {
         foreach ($data->mappings as $mapping) {
             if ($mapping->groupingid === $grouping1->id) {
                 $grouping1maps++;
-                $this->assertContains($mapping->groupid, array($group1->id, $group2->id));
+                $this->assertContainsEquals($mapping->groupid, array($group1->id, $group2->id));
             } else if ($mapping->groupingid === $grouping2->id) {
                 $grouping2maps++;
-                $this->assertContains($mapping->groupid, array($group3->id, $group4->id));
+                $this->assertContainsEquals($mapping->groupid, array($group3->id, $group4->id));
             } else {
                 $this->fail('Unexpected groupingid');
             }
@@ -525,35 +674,31 @@ class core_grouplib_testcase extends advanced_testcase {
 
         // Test the groups_get_all_groups which uses this functionality.
         $groups  = groups_get_all_groups($course->id);
-        $groupkeys = array_keys($groups);
         $this->assertCount(4, $groups);
-        $this->assertContains($group1->id, $groupkeys);
-        $this->assertContains($group2->id, $groupkeys);
-        $this->assertContains($group3->id, $groupkeys);
-        $this->assertContains($group4->id, $groupkeys);
+        $this->assertArrayHasKey($group1->id, $groups);
+        $this->assertArrayHasKey($group2->id, $groups);
+        $this->assertArrayHasKey($group3->id, $groups);
+        $this->assertArrayHasKey($group4->id, $groups);
 
         $groups  = groups_get_all_groups($course->id, null, $grouping1->id);
-        $groupkeys = array_keys($groups);
         $this->assertCount(2, $groups);
-        $this->assertContains($group1->id, $groupkeys);
-        $this->assertContains($group2->id, $groupkeys);
-        $this->assertNotContains($group3->id, $groupkeys);
-        $this->assertNotContains($group4->id, $groupkeys);
+        $this->assertArrayHasKey($group1->id, $groups);
+        $this->assertArrayHasKey($group2->id, $groups);
+        $this->assertArrayNotHasKey($group3->id, $groups);
+        $this->assertArrayNotHasKey($group4->id, $groups);
 
         $groups  = groups_get_all_groups($course->id, null, $grouping2->id);
-        $groupkeys = array_keys($groups);
         $this->assertCount(2, $groups);
-        $this->assertNotContains($group1->id, $groupkeys);
-        $this->assertNotContains($group2->id, $groupkeys);
-        $this->assertContains($group3->id, $groupkeys);
-        $this->assertContains($group4->id, $groupkeys);
+        $this->assertArrayNotHasKey($group1->id, $groups);
+        $this->assertArrayNotHasKey($group2->id, $groups);
+        $this->assertArrayHasKey($group3->id, $groups);
+        $this->assertArrayHasKey($group4->id, $groups);
 
         // Test this function using an alternate column for the result index
         $groups  = groups_get_all_groups($course->id, null, $grouping2->id, 'g.name, g.id');
-        $groupkeys = array_keys($groups);
         $this->assertCount(2, $groups);
-        $this->assertNotContains($group3->id, $groupkeys);
-        $this->assertContains($group3->name, $groupkeys);
+        $this->assertArrayNotHasKey($group3->id, $groups);
+        $this->assertArrayHasKey($group3->name, $groups);
         $this->assertEquals($group3->id, $groups[$group3->name]->id);
     }
 
@@ -570,7 +715,7 @@ class core_grouplib_testcase extends advanced_testcase {
         // Create a course category, course and groups.
         $cat = $generator->create_category(array('parent' => 0));
         $course = $generator->create_course(array('category' => $cat->id));
-        $coursecontext = context_course::instance($course->id);
+        $coursecontext = \context_course::instance($course->id);
         $group1 = $generator->create_group(array('courseid' => $course->id, 'name' => 'Group 1'));
         $group2 = $generator->create_group(array('courseid' => $course->id, 'name' => 'Group 2'));
         $group3 = $generator->create_group(array('courseid' => $course->id, 'name' => 'Group 3'));
@@ -822,7 +967,7 @@ class core_grouplib_testcase extends advanced_testcase {
 
         // Generate data.
         $course = $this->getDataGenerator()->create_course();
-        $record = new stdClass();
+        $record = new \stdClass();
         $record->courseid = $course->id;
         $group1 = $this->getDataGenerator()->create_group($record);
         $group2 = $this->getDataGenerator()->create_group($record);
@@ -839,29 +984,29 @@ class core_grouplib_testcase extends advanced_testcase {
         // Now user can access one of the group. We can't assert an exact match here because of random ids generated by yui. So do
         // partial match to see if all groups are listed or not.
         $html = groups_allgroups_course_menu($course, 'someurl.php');
-        $this->assertContains(format_string($group1->name), $html);
-        $this->assertNotContains(format_string($group2->name), $html);
+        $this->assertStringContainsString(format_string($group1->name), $html);
+        $this->assertStringNotContainsString(format_string($group2->name), $html);
 
         $this->setAdminUser();
 
         // Now user can access everything.
         $html = groups_allgroups_course_menu($course, 'someurl.php');
-        $this->assertContains(format_string($group1->name), $html);
-        $this->assertContains(format_string($group2->name), $html);
+        $this->assertStringContainsString(format_string($group1->name), $html);
+        $this->assertStringContainsString(format_string($group2->name), $html);
 
         // Make sure separate groups mode, doesn't change anything.
         $course->groupmode = SEPARATEGROUPS;
         update_course($course);
         $html = groups_allgroups_course_menu($course, 'someurl.php');
-        $this->assertContains(format_string($group1->name), $html);
-        $this->assertContains(format_string($group2->name), $html);
+        $this->assertStringContainsString(format_string($group1->name), $html);
+        $this->assertStringContainsString(format_string($group2->name), $html);
 
         // Make sure Visible groups mode, doesn't change anything.
         $course->groupmode = VISIBLEGROUPS;
         update_course($course);
         $html = groups_allgroups_course_menu($course, 'someurl.php');
-        $this->assertContains(format_string($group1->name), $html);
-        $this->assertContains(format_string($group2->name), $html);
+        $this->assertStringContainsString(format_string($group1->name), $html);
+        $this->assertStringContainsString(format_string($group2->name), $html);
 
         // Let us test activegroup changes now.
         $this->setUser($user);
@@ -907,6 +1052,76 @@ class core_grouplib_testcase extends advanced_testcase {
         $this->assertEquals('Group 2', $groups[1]->name);
     }
 
+    /**
+     * Tests for groups_get_all_groups when grouping is set and we want members as well.
+     */
+    public function test_groups_get_all_groups_in_grouping_with_members() {
+        $generator = $this->getDataGenerator();
+        $this->resetAfterTest();
+
+        // Create courses.
+        $course1 = $generator->create_course();
+        $course2 = $generator->create_course();
+
+        // Create users.
+        $c1user1 = $generator->create_user();
+        $c12user1 = $generator->create_user();
+        $c12user2 = $generator->create_user();
+
+        // Enrol users.
+        $generator->enrol_user($c1user1->id, $course1->id);
+        $generator->enrol_user($c12user1->id, $course1->id);
+        $generator->enrol_user($c12user1->id, $course2->id);
+        $generator->enrol_user($c12user2->id, $course1->id);
+        $generator->enrol_user($c12user2->id, $course2->id);
+
+        // Create groupings and groups for course1.
+        $c1grouping1 = $generator->create_grouping(array('courseid' => $course1->id));
+        $c1grouping2 = $generator->create_grouping(array('courseid' => $course1->id));
+        $c1group1 = $generator->create_group(array('courseid' => $course1->id));
+        $c1group2 = $generator->create_group(array('courseid' => $course1->id));
+        $c1group3 = $generator->create_group(array('courseid' => $course1->id));
+        groups_assign_grouping($c1grouping1->id, $c1group1->id);
+        groups_assign_grouping($c1grouping1->id, $c1group2->id);
+        groups_assign_grouping($c1grouping2->id, $c1group3->id);
+
+        // Create groupings and groups for course2.
+        $c2grouping1 = $generator->create_grouping(array('courseid' => $course2->id));
+        $c2group1 = $generator->create_group(array('courseid' => $course1->id));
+        groups_assign_grouping($c2grouping1->id, $c2group1->id);
+
+        // Assign users to groups.
+        $generator->create_group_member(array('groupid' => $c1group1->id, 'userid' => $c1user1->id));
+        $generator->create_group_member(array('groupid' => $c1group1->id, 'userid' => $c12user1->id));
+        $generator->create_group_member(array('groupid' => $c1group2->id, 'userid' => $c12user2->id));
+        $generator->create_group_member(array('groupid' => $c2group1->id, 'userid' => $c12user2->id));
+
+        // Test without userid.
+        $groups = groups_get_all_groups($course1->id, null, $c1grouping1->id, 'g.*', true);
+
+        $this->assertEqualsCanonicalizing(
+                [$c1group1->id, $c1group2->id],
+                array_keys($groups)
+        );
+        $this->assertEquals(
+                [$c1user1->id => $c1user1->id, $c12user1->id => $c12user1->id],
+                $groups[$c1group1->id]->members
+        );
+        $this->assertEquals(
+                [$c12user2->id => $c12user2->id],
+                $groups[$c1group2->id]->members
+        );
+
+        // Test with userid.
+        $groups = groups_get_all_groups($course1->id, $c1user1->id, $c1grouping1->id, 'g.*', true);
+
+        $this->assertEquals([$c1group1->id], array_keys($groups));
+        $this->assertEqualsCanonicalizing(
+                [$c1user1->id, $c12user1->id],
+                $groups[$c1group1->id]->members
+        );
+    }
+
     /**
      * Tests for groups_get_user_groups() method.
      */
@@ -986,7 +1201,7 @@ class core_grouplib_testcase extends advanced_testcase {
     protected function make_group_list($number) {
         $testgroups = array();
         for ($a = 0; $a < $number; $a++) {
-            $grp = new stdClass();
+            $grp = new \stdClass();
             $grp->id = 100 + $a;
             $grp->name = 'test group ' . $grp->id;
             $testgroups[$grp->id] = $grp;
@@ -1051,7 +1266,7 @@ class core_grouplib_testcase extends advanced_testcase {
      * Tests for groups_user_groups_visible.
      */
     public function test_groups_user_groups_visible() {
-        global $CFG, $DB;
+        global $DB;
 
         $generator = $this->getDataGenerator();
         $this->resetAfterTest();
@@ -1060,7 +1275,7 @@ class core_grouplib_testcase extends advanced_testcase {
         // Create a course category, course and groups.
         $cat = $generator->create_category(array('parent' => 0));
         $course = $generator->create_course(array('category' => $cat->id));
-        $coursecontext = context_course::instance($course->id);
+        $coursecontext = \context_course::instance($course->id);
         $group1 = $generator->create_group(array('courseid' => $course->id, 'name' => 'Group 1'));
         $group2 = $generator->create_group(array('courseid' => $course->id, 'name' => 'Group 2'));
         $group3 = $generator->create_group(array('courseid' => $course->id, 'name' => 'Group 3'));
@@ -1100,7 +1315,8 @@ class core_grouplib_testcase extends advanced_testcase {
         $result = groups_user_groups_visible($course, $user2->id);
         $this->assertTrue($result);
 
-        $result = groups_user_groups_visible($course, $user2->id);
+        $cm->groupmode = NOGROUPS;
+        $result = groups_user_groups_visible($course, $user2->id, $cm);
         $this->assertTrue($result); // Cm with no groups.
 
         $cm->groupmode = SEPARATEGROUPS;
@@ -1118,7 +1334,8 @@ class core_grouplib_testcase extends advanced_testcase {
         $result = groups_user_groups_visible($course, $user2->id);
         $this->assertTrue($result);
 
-        $result = groups_user_groups_visible($course, $user2->id);
+        $cm->groupmode = NOGROUPS;
+        $result = groups_user_groups_visible($course, $user2->id, $cm);
         $this->assertTrue($result); // Cm with no groups.
 
         $cm->groupmode = SEPARATEGROUPS;
@@ -1156,7 +1373,7 @@ class core_grouplib_testcase extends advanced_testcase {
         $this->assertTrue($result);
 
         $cm->groupmode = NOGROUPS;
-        $result = groups_user_groups_visible($course, $user2->id);
+        $result = groups_user_groups_visible($course, $user2->id, $cm);
         $this->assertTrue($result); // Cm with no groups.
 
         $cm->groupmode = SEPARATEGROUPS;
@@ -1224,7 +1441,8 @@ class core_grouplib_testcase extends advanced_testcase {
         $result = groups_user_groups_visible($course, $user4->id);
         $this->assertTrue($result);
 
-        $result = groups_user_groups_visible($course, $user4->id);
+        $cm->groupmode = NOGROUPS;
+        $result = groups_user_groups_visible($course, $user4->id, $cm);
         $this->assertTrue($result); // Cm with no groups.
 
         $cm->groupmode = SEPARATEGROUPS;
@@ -1242,7 +1460,8 @@ class core_grouplib_testcase extends advanced_testcase {
         $result = groups_user_groups_visible($course, $user4->id);
         $this->assertTrue($result);
 
-        $result = groups_user_groups_visible($course, $user4->id);
+        $cm->groupmode = NOGROUPS;
+        $result = groups_user_groups_visible($course, $user4->id, $cm);
         $this->assertTrue($result); // Cm with no groups.
 
         $cm->groupmode = SEPARATEGROUPS;
@@ -1250,7 +1469,7 @@ class core_grouplib_testcase extends advanced_testcase {
         $this->assertTrue($result); // Cm with separate groups.
 
         $cm->groupmode = SEPARATEGROUPS;
-        $result = groups_user_groups_visible($course, $user4->id);
+        $result = groups_user_groups_visible($course, $user4->id, $cm);
         $this->assertTrue($result); // Cm with visible groups.
 
         // Visible groups, forced.
@@ -1280,7 +1499,7 @@ class core_grouplib_testcase extends advanced_testcase {
         $this->assertTrue($result);
 
         $cm->groupmode = NOGROUPS;
-        $result = groups_user_groups_visible($course, $user4->id);
+        $result = groups_user_groups_visible($course, $user4->id, $cm);
         $this->assertTrue($result); // Cm with no groups.
 
         $cm->groupmode = SEPARATEGROUPS;
@@ -1352,7 +1571,8 @@ class core_grouplib_testcase extends advanced_testcase {
         $result = groups_user_groups_visible($course, $user1->id);
         $this->assertTrue($result); // Requesting all groups.
 
-        $result = groups_user_groups_visible($course, $user1->id);
+        $cm->groupmode = NOGROUPS;
+        $result = groups_user_groups_visible($course, $user1->id, $cm);
         $this->assertTrue($result); // Cm with no groups.
 
         $cm->groupmode = SEPARATEGROUPS;
@@ -1374,7 +1594,8 @@ class core_grouplib_testcase extends advanced_testcase {
         $result = groups_user_groups_visible($course, $user1->id);
         $this->assertTrue($result); // Requesting all groups.
 
-        $result = groups_user_groups_visible($course, $user1->id);
+        $cm->groupmode = NOGROUPS;
+        $result = groups_user_groups_visible($course, $user1->id, $cm);
         $this->assertTrue($result); // Cm with no groups.
 
         $cm->groupmode = SEPARATEGROUPS;
@@ -1384,7 +1605,7 @@ class core_grouplib_testcase extends advanced_testcase {
         $this->assertTrue($result); // Cm with separate groups.
 
         $cm->groupmode = SEPARATEGROUPS;
-        $result = groups_user_groups_visible($course, $user1->id);
+        $result = groups_user_groups_visible($course, $user1->id, $cm);
         $this->assertTrue($result); // Cm with visible groups.
 
         // Visible groups, forced.
@@ -1420,7 +1641,7 @@ class core_grouplib_testcase extends advanced_testcase {
         $this->assertTrue($result); // Requesting all groups.
 
         $cm->groupmode = NOGROUPS;
-        $result = groups_user_groups_visible($course, $user1->id);
+        $result = groups_user_groups_visible($course, $user1->id, $cm);
         $this->assertTrue($result); // Cm with no groups.
 
         $cm->groupmode = SEPARATEGROUPS;
@@ -1614,12 +1835,12 @@ class core_grouplib_testcase extends advanced_testcase {
         // Retrieve users sharing groups with user1.
         $members = groups_get_activity_shared_group_members($cm, $user1->id);
         $this->assertCount(2, $members);
-        $this->assertEquals([$user1->id, $user2->id], array_keys($members), '', 0.0, 10, true);
+        $this->assertEqualsCanonicalizing([$user1->id, $user2->id], array_keys($members));
 
         // Retrieve users sharing groups with user2.
         $members = groups_get_activity_shared_group_members($cm, $user2->id);
         $this->assertCount(2, $members);
-        $this->assertEquals([$user1->id, $user2->id], array_keys($members), '', 0.0, 10, true);
+        $this->assertEqualsCanonicalizing([$user1->id, $user2->id], array_keys($members));
 
         // Retrieve users sharing groups with user3.
         $members = groups_get_activity_shared_group_members($cm, $user3->id);
@@ -1648,170 +1869,6 @@ class core_grouplib_testcase extends advanced_testcase {
         $generator->create_group_member(array('groupid' => $group3->id, 'userid' => $user1->id));
         $members = groups_get_activity_shared_group_members($cm, $user1->id);
         $this->assertCount(2, $members);    // Now I see members of group 3.
-        $this->assertEquals([$user1->id, $user3->id], array_keys($members), '', 0.0, 10, true);
-    }
-
-    /**
-     * Test groups_get_all_groups_for_courses() method.
-     */
-    public function test_groups_get_all_groups_for_courses_no_courses() {
-        $this->resetAfterTest(true);
-        $generator = $this->getDataGenerator();
-
-        $this->assertEquals([], groups_get_all_groups_for_courses([]));
-    }
-
-    /**
-     * Test groups_get_all_groups_for_courses() method.
-     */
-    public function test_groups_get_all_groups_for_courses_with_courses() {
-        global $DB;
-
-        $this->resetAfterTest(true);
-        $generator = $this->getDataGenerator();
-
-        // Create courses.
-        $course1 = $generator->create_course(); // no groups.
-        $course2 = $generator->create_course(); // one group, no members.
-        $course3 = $generator->create_course(); // one group, one member.
-        $course4 = $generator->create_course(); // one group, multiple members.
-        $course5 = $generator->create_course(); // two groups, no members.
-        $course6 = $generator->create_course(); // two groups, one member.
-        $course7 = $generator->create_course(); // two groups, multiple members.
-
-        $courses = [$course1, $course2, $course3, $course4, $course5, $course6, $course7];
-        // Create users.
-        $user1 = $generator->create_user();
-        $user2 = $generator->create_user();
-        $user3 = $generator->create_user();
-        $user4 = $generator->create_user();
-
-        // Enrol users.
-        foreach ($courses as $course) {
-            $generator->enrol_user($user1->id, $course->id);
-            $generator->enrol_user($user2->id, $course->id);
-            $generator->enrol_user($user3->id, $course->id);
-            $generator->enrol_user($user4->id, $course->id);
-        }
-
-        // Create groups.
-        $group1 = $generator->create_group(array('courseid' => $course2->id)); // no members.
-        $group2 = $generator->create_group(array('courseid' => $course3->id)); // one member.
-        $group3 = $generator->create_group(array('courseid' => $course4->id)); // multiple members.
-        $group4 = $generator->create_group(array('courseid' => $course5->id)); // no members.
-        $group5 = $generator->create_group(array('courseid' => $course5->id)); // no members.
-        $group6 = $generator->create_group(array('courseid' => $course6->id)); // one member.
-        $group7 = $generator->create_group(array('courseid' => $course6->id)); // one member.
-        $group8 = $generator->create_group(array('courseid' => $course7->id)); // multiple members.
-        $group9 = $generator->create_group(array('courseid' => $course7->id)); // multiple members.
-
-        // Assign users to groups.
-        $generator->create_group_member(array('groupid' => $group2->id, 'userid' => $user1->id));
-        $generator->create_group_member(array('groupid' => $group3->id, 'userid' => $user1->id));
-        $generator->create_group_member(array('groupid' => $group3->id, 'userid' => $user2->id));
-        $generator->create_group_member(array('groupid' => $group6->id, 'userid' => $user1->id));
-        $generator->create_group_member(array('groupid' => $group7->id, 'userid' => $user1->id));
-        $generator->create_group_member(array('groupid' => $group8->id, 'userid' => $user1->id));
-        $generator->create_group_member(array('groupid' => $group8->id, 'userid' => $user2->id));
-        $generator->create_group_member(array('groupid' => $group9->id, 'userid' => $user1->id));
-        $generator->create_group_member(array('groupid' => $group9->id, 'userid' => $user2->id));
-
-        // The process of modifying group members changes the timemodified of the group.
-        // Refresh the group records.
-        $group1 = $DB->get_record('groups', ['id' => $group1->id]);
-        $group2 = $DB->get_record('groups', ['id' => $group2->id]);
-        $group3 = $DB->get_record('groups', ['id' => $group3->id]);
-        $group4 = $DB->get_record('groups', ['id' => $group4->id]);
-        $group5 = $DB->get_record('groups', ['id' => $group5->id]);
-        $group6 = $DB->get_record('groups', ['id' => $group6->id]);
-        $group7 = $DB->get_record('groups', ['id' => $group7->id]);
-        $group8 = $DB->get_record('groups', ['id' => $group8->id]);
-        $group9 = $DB->get_record('groups', ['id' => $group9->id]);
-
-        $result = groups_get_all_groups_for_courses($courses);
-        $assertpropertiesmatch = function($expected, $actual) {
-            $props = get_object_vars($expected);
-
-            foreach ($props as $name => $val) {
-                $got = $actual->{$name};
-                $this->assertEquals(
-                    $val,
-                    $actual->{$name},
-                    "Failed asserting that {$got} equals {$val} for property {$name}"
-                );
-            }
-        };
-
-        // Course 1 has no groups.
-        $this->assertEquals([], $result[$course1->id]);
-
-        // Course 2 has one group with no members.
-        $coursegroups = $result[$course2->id];
-        $coursegroup = $coursegroups[$group1->id];
-        $this->assertCount(1, $coursegroups);
-        $this->assertEquals([], $coursegroup->members);
-        $assertpropertiesmatch($group1, $coursegroup);
-
-        // Course 3 has one group with one member.
-        $coursegroups = $result[$course3->id];
-        $coursegroup = $coursegroups[$group2->id];
-        $groupmember1 = $coursegroup->members[$user1->id];
-        $this->assertCount(1, $coursegroups);
-        $this->assertCount(1, $coursegroup->members);
-        $assertpropertiesmatch($group2, $coursegroup);
-        $this->assertEquals($user1->id, $groupmember1->userid);
-
-        // Course 4 has one group with multiple members.
-        $coursegroups = $result[$course4->id];
-        $coursegroup = $coursegroups[$group3->id];
-        $groupmember1 = $coursegroup->members[$user1->id];
-        $groupmember2 = $coursegroup->members[$user2->id];
-        $this->assertCount(1, $coursegroups);
-        $this->assertCount(2, $coursegroup->members);
-        $assertpropertiesmatch($group3, $coursegroup);
-        $this->assertEquals($user1->id, $groupmember1->userid);
-        $this->assertEquals($user2->id, $groupmember2->userid);
-
-        // Course 5 has multiple groups with no members.
-        $coursegroups = $result[$course5->id];
-        $coursegroup1 = $coursegroups[$group4->id];
-        $coursegroup2 = $coursegroups[$group5->id];
-        $this->assertCount(2, $coursegroups);
-        $this->assertEquals([], $coursegroup1->members);
-        $this->assertEquals([], $coursegroup2->members);
-        $assertpropertiesmatch($group4, $coursegroup1);
-        $assertpropertiesmatch($group5, $coursegroup2);
-
-        // Course 6 has multiple groups with one member.
-        $coursegroups = $result[$course6->id];
-        $coursegroup1 = $coursegroups[$group6->id];
-        $coursegroup2 = $coursegroups[$group7->id];
-        $group1member1 = $coursegroup1->members[$user1->id];
-        $group2member1 = $coursegroup2->members[$user1->id];
-        $this->assertCount(2, $coursegroups);
-        $this->assertCount(1, $coursegroup1->members);
-        $this->assertCount(1, $coursegroup2->members);
-        $assertpropertiesmatch($group6, $coursegroup1);
-        $assertpropertiesmatch($group7, $coursegroup2);
-        $this->assertEquals($user1->id, $group1member1->userid);
-        $this->assertEquals($user1->id, $group2member1->userid);
-
-        // Course 7 has multiple groups with multiple members.
-        $coursegroups = $result[$course7->id];
-        $coursegroup1 = $coursegroups[$group8->id];
-        $coursegroup2 = $coursegroups[$group9->id];
-        $group1member1 = $coursegroup1->members[$user1->id];
-        $group1member2 = $coursegroup1->members[$user2->id];
-        $group2member1 = $coursegroup2->members[$user1->id];
-        $group2member2 = $coursegroup2->members[$user2->id];
-        $this->assertCount(2, $coursegroups);
-        $this->assertCount(2, $coursegroup1->members);
-        $this->assertCount(2, $coursegroup2->members);
-        $assertpropertiesmatch($group8, $coursegroup1);
-        $assertpropertiesmatch($group9, $coursegroup2);
-        $this->assertEquals($user1->id, $group1member1->userid);
-        $this->assertEquals($user2->id, $group1member2->userid);
-        $this->assertEquals($user1->id, $group2member1->userid);
-        $this->assertEquals($user2->id, $group2member2->userid);
+        $this->assertEqualsCanonicalizing([$user1->id, $user3->id], array_keys($members));
     }
 }
