@@ -14,26 +14,28 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
+namespace core;
+
+use core_media_manager;
+use media_test_plugin;
+
+defined('MOODLE_INTERNAL') || die();
+require_once(__DIR__ . '/fixtures/testable_core_media_player.php');
+
 /**
  * Test classes for handling embedded media (audio/video).
  *
- * @package core_media
- * @category phpunit
+ * @package   core
+ * @category  test
  * @copyright 2012 The Open University
  * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-
-defined('MOODLE_INTERNAL') || die();
-
-/**
- * Test script for media embedding.
- */
-class core_medialib_testcase extends advanced_testcase {
+class medialib_test extends \advanced_testcase {
 
     /**
      * Pre-test setup. Preserves $CFG.
      */
-    public function setUp() {
+    public function setUp(): void {
         parent::setUp();
 
         // Reset $CFG and $SERVER.
@@ -54,7 +56,7 @@ class core_medialib_testcase extends advanced_testcase {
      */
     private function pretend_to_be_safari() {
         // Pretend to be using Safari browser (must support mp4 for tests to work).
-        core_useragent::instance(true, 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-at) ' .
+        \core_useragent::instance(true, 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-at) ' .
                 'AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1');
     }
 
@@ -63,7 +65,7 @@ class core_medialib_testcase extends advanced_testcase {
      */
     private function pretend_to_be_firefox() {
         // Pretend to be using Firefox browser (must support ogg for tests to work).
-        core_useragent::instance(true, 'Mozilla/5.0 (X11; Linux x86_64; rv:46.0) Gecko/20100101 Firefox/46.0 ');
+        \core_useragent::instance(true, 'Mozilla/5.0 (X11; Linux x86_64; rv:46.0) Gecko/20100101 Firefox/46.0 ');
     }
 
     /**
@@ -72,11 +74,11 @@ class core_medialib_testcase extends advanced_testcase {
     public function test_get_filename() {
         $manager = core_media_manager::instance();
 
-        $this->assertSame('frog.mp4', $manager->get_filename(new moodle_url(
+        $this->assertSame('frog.mp4', $manager->get_filename(new \moodle_url(
                 '/pluginfile.php/312/mod_page/content/7/frog.mp4')));
         // This should work even though slasharguments is true, because we want
         // it to support 'legacy' links if somebody toggles the option later.
-        $this->assertSame('frog.mp4', $manager->get_filename(new moodle_url(
+        $this->assertSame('frog.mp4', $manager->get_filename(new \moodle_url(
                 '/pluginfile.php?file=/312/mod_page/content/7/frog.mp4')));
     }
 
@@ -86,13 +88,13 @@ class core_medialib_testcase extends advanced_testcase {
     public function test_get_extension() {
         $manager = core_media_manager::instance();
 
-        $this->assertSame('mp4', $manager->get_extension(new moodle_url(
+        $this->assertSame('mp4', $manager->get_extension(new \moodle_url(
                 '/pluginfile.php/312/mod_page/content/7/frog.mp4')));
-        $this->assertSame('', $manager->get_extension(new moodle_url(
+        $this->assertSame('', $manager->get_extension(new \moodle_url(
                 '/pluginfile.php/312/mod_page/content/7/frog')));
-        $this->assertSame('mp4', $manager->get_extension(new moodle_url(
+        $this->assertSame('mp4', $manager->get_extension(new \moodle_url(
                 '/pluginfile.php?file=/312/mod_page/content/7/frog.mp4')));
-        $this->assertSame('', $manager->get_extension(new moodle_url(
+        $this->assertSame('', $manager->get_extension(new \moodle_url(
                 '/pluginfile.php?file=/312/mod_page/content/7/frog')));
     }
 
@@ -103,9 +105,9 @@ class core_medialib_testcase extends advanced_testcase {
         $test = new media_test_plugin(1, 13, ['tst', 'test']);
 
         // Some example URLs.
-        $supported1 = new moodle_url('http://example.org/1.test');
-        $supported2 = new moodle_url('http://example.org/2.TST');
-        $unsupported = new moodle_url('http://example.org/2.jpg');
+        $supported1 = new \moodle_url('http://example.org/1.test');
+        $supported2 = new \moodle_url('http://example.org/2.TST');
+        $unsupported = new \moodle_url('http://example.org/2.jpg');
 
         // No URLs => none.
         $result = $test->list_supported_urls(array());
@@ -129,40 +131,40 @@ class core_medialib_testcase extends advanced_testcase {
     }
 
     /**
-     * Test for core_media_renderer get_players
+     * Test for get_players
      */
     public function test_get_players() {
         // All players are initially disabled (except link, which you can't).
-        $manager = new core_media_manager_test();
-        $this->assertEmpty($manager->get_players_test());
+        $manager = core_media_manager::instance();
+        $this->assertEmpty($this->get_players_test($manager));
 
         // A couple enabled, check the order.
         \core\plugininfo\media::set_enabled_plugins('youtube,html5audio');
-        $manager = new core_media_manager_test();
-        $this->assertSame('youtube, html5audio', $manager->get_players_test());
+        $manager = core_media_manager::instance();
+        $this->assertSame('youtube, html5audio', $this->get_players_test($manager));
 
         // Test SWF and HTML5 media order.
         \core\plugininfo\media::set_enabled_plugins('html5video,html5audio,swf');
-        $manager = new core_media_manager_test();
-        $this->assertSame('html5video, html5audio, swf', $manager->get_players_test());
+        $manager = core_media_manager::instance();
+        $this->assertSame('html5video, html5audio, swf', $this->get_players_test($manager));
 
         // Make sure that our test plugin is considered installed.
         \core\plugininfo\media::set_enabled_plugins('test,html5video');
-        $manager = new core_media_manager_test();
-        $this->assertSame('test, html5video', $manager->get_players_test());
+        $manager = core_media_manager::instance();
+        $this->assertSame('test, html5video', $this->get_players_test($manager));
 
         // Make sure that non-existing plugin is NOT considered installed.
         \core\plugininfo\media::set_enabled_plugins('nonexistingplugin,html5video');
-        $manager = new core_media_manager_test();
-        $this->assertSame('html5video', $manager->get_players_test());
+        $manager = core_media_manager::instance();
+        $this->assertSame('html5video', $this->get_players_test($manager));
     }
 
     /**
-     * Test for core_media_renderer can_embed_url
+     * Test for can_embed_url
      */
     public function test_can_embed_url() {
         // All players are initially disabled, so mp4 cannot be rendered.
-        $url = new moodle_url('http://example.org/test.mp4');
+        $url = new \moodle_url('http://example.org/test.mp4');
         $manager = core_media_manager::instance();
         $this->assertFalse($manager->can_embed_url($url));
 
@@ -188,7 +190,7 @@ class core_medialib_testcase extends advanced_testcase {
     }
 
     /**
-     * Test for core_media_renderer embed_url.
+     * Test for embed_url.
      * Checks multiple format/fallback support.
      */
     public function test_embed_url_fallbacks() {
@@ -200,7 +202,7 @@ class core_medialib_testcase extends advanced_testcase {
         $link = 'mediafallbacklink';
         $test = 'mediaplugin_test';
 
-        $url = new moodle_url('http://example.org/test.mp4');
+        $url = new \moodle_url('http://example.org/test.mp4');
 
         // All plugins disabled, NOLINK option.
         \core\plugininfo\media::set_enabled_plugins('');
@@ -214,7 +216,7 @@ class core_medialib_testcase extends advanced_testcase {
         \core\plugininfo\media::set_enabled_plugins('');
         $manager = core_media_manager::instance();
         $t = $manager->embed_url($url);
-        $this->assertContains($link, $t);
+        $this->assertStringContainsString($link, $t);
 
         // Enable media players that can play the same media formats. (ie. test & html5audio for mp3 files, etc.)
         \core\plugininfo\media::set_enabled_plugins('test,html5video,html5audio,swf');
@@ -224,37 +226,37 @@ class core_medialib_testcase extends advanced_testcase {
         $mediaformats = array('mp3', 'mp4');
 
         foreach ($mediaformats as $format) {
-            $url = new moodle_url('http://example.org/test.' . $format);
+            $url = new \moodle_url('http://example.org/test.' . $format);
             $textwithlink = $manager->embed_url($url);
             $textwithoutlink = $manager->embed_url($url, 0, 0, '', array(core_media_manager::OPTION_NO_LINK => true));
 
             switch ($format) {
                 case 'mp3':
-                    $this->assertContains($test, $textwithlink);
-                    $this->assertNotContains($html5video, $textwithlink);
-                    $this->assertContains($html5audio, $textwithlink);
-                    $this->assertNotContains($swf, $textwithlink);
-                    $this->assertContains($link, $textwithlink);
+                    $this->assertStringContainsString($test, $textwithlink);
+                    $this->assertStringNotContainsString($html5video, $textwithlink);
+                    $this->assertStringContainsString($html5audio, $textwithlink);
+                    $this->assertStringNotContainsString($swf, $textwithlink);
+                    $this->assertStringContainsString($link, $textwithlink);
 
-                    $this->assertContains($test, $textwithoutlink);
-                    $this->assertNotContains($html5video, $textwithoutlink);
-                    $this->assertContains($html5audio, $textwithoutlink);
-                    $this->assertNotContains($swf, $textwithoutlink);
-                    $this->assertNotContains($link, $textwithoutlink);
+                    $this->assertStringContainsString($test, $textwithoutlink);
+                    $this->assertStringNotContainsString($html5video, $textwithoutlink);
+                    $this->assertStringContainsString($html5audio, $textwithoutlink);
+                    $this->assertStringNotContainsString($swf, $textwithoutlink);
+                    $this->assertStringNotContainsString($link, $textwithoutlink);
                     break;
 
                 case 'mp4':
-                    $this->assertContains($test, $textwithlink);
-                    $this->assertContains($html5video, $textwithlink);
-                    $this->assertNotContains($html5audio, $textwithlink);
-                    $this->assertNotContains($swf, $textwithlink);
-                    $this->assertContains($link, $textwithlink);
+                    $this->assertStringContainsString($test, $textwithlink);
+                    $this->assertStringContainsString($html5video, $textwithlink);
+                    $this->assertStringNotContainsString($html5audio, $textwithlink);
+                    $this->assertStringNotContainsString($swf, $textwithlink);
+                    $this->assertStringContainsString($link, $textwithlink);
 
-                    $this->assertContains($test, $textwithoutlink);
-                    $this->assertContains($html5video, $textwithoutlink);
-                    $this->assertNotContains($html5audio, $textwithoutlink);
-                    $this->assertNotContains($swf, $textwithoutlink);
-                    $this->assertNotContains($link, $textwithoutlink);
+                    $this->assertStringContainsString($test, $textwithoutlink);
+                    $this->assertStringContainsString($html5video, $textwithoutlink);
+                    $this->assertStringNotContainsString($html5audio, $textwithoutlink);
+                    $this->assertStringNotContainsString($swf, $textwithoutlink);
+                    $this->assertStringNotContainsString($link, $textwithoutlink);
                     break;
 
                 default:
@@ -264,7 +266,7 @@ class core_medialib_testcase extends advanced_testcase {
     }
 
     /**
-     * Test for core_media_renderer embed_url.
+     * Test for embed_url.
      * Check SWF works including the special option required to enable it
      */
     public function test_embed_url_swf() {
@@ -272,14 +274,14 @@ class core_medialib_testcase extends advanced_testcase {
         $manager = core_media_manager::instance();
 
         // Without any options...
-        $url = new moodle_url('http://example.org/test.swf');
+        $url = new \moodle_url('http://example.org/test.swf');
         $t = $manager->embed_url($url);
-        $this->assertNotContains('</object>', $t);
+        $this->assertStringNotContainsString('</object>', $t);
 
         // ...and with the 'no it's safe, I checked it' option.
-        $url = new moodle_url('http://example.org/test.swf');
+        $url = new \moodle_url('http://example.org/test.swf');
         $t = $manager->embed_url($url, '', 0, 0, array(core_media_manager::OPTION_TRUSTED => true));
-        $this->assertContains('</object>', $t);
+        $this->assertStringContainsString('</object>', $t);
     }
 
     /**
@@ -296,13 +298,13 @@ class core_medialib_testcase extends advanced_testcase {
         $manager = core_media_manager::instance();
 
         // Format: mp3.
-        $url = new moodle_url('http://example.org/pluginfile.php?file=x/y/z/test.mp3');
+        $url = new \moodle_url('http://example.org/pluginfile.php?file=x/y/z/test.mp3');
         $t = $manager->embed_url($url);
-        $this->assertContains('</audio>', $t);
+        $this->assertStringContainsString('</audio>', $t);
     }
 
     /**
-     * Test for core_media_renderer embed_url.
+     * Test for embed_url.
      * Checks the EMBED_OR_BLANK option.
      */
     public function test_embed_or_blank() {
@@ -313,19 +315,19 @@ class core_medialib_testcase extends advanced_testcase {
         $options = array(core_media_manager::OPTION_FALLBACK_TO_BLANK => true);
 
         // Embed that does match something should still include the link too.
-        $url = new moodle_url('http://example.org/test.ogg');
+        $url = new \moodle_url('http://example.org/test.ogg');
         $t = $manager->embed_url($url, '', 0, 0, $options);
-        $this->assertContains('</audio>', $t);
-        $this->assertContains('mediafallbacklink', $t);
+        $this->assertStringContainsString('</audio>', $t);
+        $this->assertStringContainsString('mediafallbacklink', $t);
 
         // Embed that doesn't match something should be totally blank.
-        $url = new moodle_url('http://example.org/test.mp4');
+        $url = new \moodle_url('http://example.org/test.mp4');
         $t = $manager->embed_url($url, '', 0, 0, $options);
         $this->assertSame('', $t);
     }
 
     /**
-     * Test for core_media_renderer embed_url.
+     * Test for embed_url.
      * Checks that size is passed through correctly to player objects and tests
      * size support in html5video output.
      */
@@ -338,27 +340,27 @@ class core_medialib_testcase extends advanced_testcase {
         // through.
         \core\plugininfo\media::set_enabled_plugins('html5video');
         $manager = core_media_manager::instance();
-        $url = new moodle_url('http://example.org/test.mp4');
+        $url = new \moodle_url('http://example.org/test.mp4');
 
         // HTML5 default size - specifies core width and does not specify height.
         $t = $manager->embed_url($url);
-        $this->assertContains('width="' . $CFG->media_default_width . '"', $t);
-        $this->assertNotContains('height', $t);
+        $this->assertStringContainsString('width="' . $CFG->media_default_width . '"', $t);
+        $this->assertStringNotContainsString('height', $t);
 
         // HTML5 specified size - specifies both.
         $t = $manager->embed_url($url, '', '666', '101');
-        $this->assertContains('width="666"', $t);
-        $this->assertContains('height="101"', $t);
+        $this->assertStringContainsString('width="666"', $t);
+        $this->assertStringContainsString('height="101"', $t);
 
         // HTML5 size specified in url, overrides call.
-        $url = new moodle_url('http://example.org/test.mp4?d=123x456');
+        $url = new \moodle_url('http://example.org/test.mp4?d=123x456');
         $t = $manager->embed_url($url, '', '666', '101');
-        $this->assertContains('width="123"', $t);
-        $this->assertContains('height="456"', $t);
+        $this->assertStringContainsString('width="123"', $t);
+        $this->assertStringContainsString('height="456"', $t);
     }
 
     /**
-     * Test for core_media_renderer embed_url.
+     * Test for embed_url.
      * Checks that name is passed through correctly to player objects and tests
      * name support in html5video output.
      */
@@ -367,19 +369,19 @@ class core_medialib_testcase extends advanced_testcase {
         // html5video.
         \core\plugininfo\media::set_enabled_plugins('html5video');
         $manager = core_media_manager::instance();
-        $url = new moodle_url('http://example.org/test.mp4');
+        $url = new \moodle_url('http://example.org/test.mp4');
 
         // HTML5 default name - use filename.
         $t = $manager->embed_url($url);
-        $this->assertContains('title="test.mp4"', $t);
+        $this->assertStringContainsString('title="test.mp4"', $t);
 
         // HTML5 specified name - check escaping.
         $t = $manager->embed_url($url, 'frog & toad');
-        $this->assertContains('title="frog &amp; toad"', $t);
+        $this->assertStringContainsString('title="frog &amp; toad"', $t);
     }
 
     /**
-     * Test for core_media_renderer split_alternatives.
+     * Test for split_alternatives.
      */
     public function test_split_alternatives() {
         $mediamanager = core_media_manager::instance();
@@ -416,7 +418,7 @@ class core_medialib_testcase extends advanced_testcase {
     }
 
     /**
-     * Test for core_media_renderer embed_alternatives (with multiple urls)
+     * Test for embed_alternatives (with multiple urls)
      */
     public function test_embed_alternatives() {
         // Most aspects of this are same as single player so let's just try
@@ -424,10 +426,10 @@ class core_medialib_testcase extends advanced_testcase {
 
         // MP4, OGV, WebM and FLV.
         $urls = array(
-            new moodle_url('http://example.org/test.mp4'),
-            new moodle_url('http://example.org/test.ogv'),
-            new moodle_url('http://example.org/test.webm'),
-            new moodle_url('http://example.org/test.flv'),
+            new \moodle_url('http://example.org/test.mp4'),
+            new \moodle_url('http://example.org/test.ogv'),
+            new \moodle_url('http://example.org/test.webm'),
+            new \moodle_url('http://example.org/test.flv'),
         );
 
         // Enable html5 and "test" ("test" first).
@@ -438,10 +440,10 @@ class core_medialib_testcase extends advanced_testcase {
         $t = $manager->embed_alternatives($urls);
 
         // HTML5 sources - mp4, but not ogv, flv or webm (not supported in Safari).
-        $this->assertContains('<source src="http://example.org/test.mp4"', $t);
-        $this->assertNotContains('<source src="http://example.org/test.ogv"', $t);
-        $this->assertNotContains('<source src="http://example.org/test.webm"', $t);
-        $this->assertNotContains('<source src="http://example.org/test.flv"', $t);
+        $this->assertStringContainsString('<source src="http://example.org/test.mp4"', $t);
+        $this->assertStringNotContainsString('<source src="http://example.org/test.ogv"', $t);
+        $this->assertStringNotContainsString('<source src="http://example.org/test.webm"', $t);
+        $this->assertStringNotContainsString('<source src="http://example.org/test.flv"', $t);
 
         // FLV is before the video tag (indicating html5 is used as fallback to flv
         // and not vice versa).
@@ -452,62 +454,40 @@ class core_medialib_testcase extends advanced_testcase {
         $t = $manager->embed_alternatives($urls);
 
         // HTML5 sources - mp4, ogv and webm, but not flv.
-        $this->assertContains('<source src="http://example.org/test.mp4"', $t);
-        $this->assertContains('<source src="http://example.org/test.ogv"', $t);
-        $this->assertContains('<source src="http://example.org/test.webm"', $t);
-        $this->assertNotContains('<source src="http://example.org/test.flv"', $t);
+        $this->assertStringContainsString('<source src="http://example.org/test.mp4"', $t);
+        $this->assertStringContainsString('<source src="http://example.org/test.ogv"', $t);
+        $this->assertStringContainsString('<source src="http://example.org/test.webm"', $t);
+        $this->assertStringNotContainsString('<source src="http://example.org/test.flv"', $t);
     }
-}
-
-/**
- * Media player stub for testing purposes.
- */
-class media_test_plugin extends core_media_player {
-    /** @var array Array of supported extensions */
-    public $ext;
-    /** @var int Player rank */
-    public $rank;
-    /** @var int Arbitrary number */
-    public $num;
 
     /**
-     * @param int $num Number (used in output)
-     * @param int $rank Player rank
-     * @param array $ext Array of supported extensions
+     * Make sure the instance() method returns singleton for the same page and different object for another page
      */
-    public function __construct($num = 1, $rank = 13, $ext = array('mp3', 'flv', 'f4v', 'mp4')) {
-        $this->ext = $ext;
-        $this->rank = $rank;
-        $this->num = $num;
+    public function test_initialise() {
+        $moodlepage1 = new \moodle_page();
+
+        $mediamanager1 = core_media_manager::instance($moodlepage1);
+        $mediamanager2 = core_media_manager::instance($moodlepage1);
+
+        $this->assertSame($mediamanager1, $mediamanager2);
+
+        $moodlepage3 = new \moodle_page();
+        $mediamanager3 = core_media_manager::instance($moodlepage3);
+
+        $this->assertNotSame($mediamanager1, $mediamanager3);
     }
 
-    public function embed($urls, $name, $width, $height, $options) {
-        self::pick_video_size($width, $height);
-        $contents = "\ntestsource=". join("\ntestsource=", $urls) .
-            "\ntestname=$name\ntestwidth=$width\ntestheight=$height\n<!--FALLBACK-->\n";
-        return html_writer::span($contents, 'mediaplugin mediaplugin_test');
-    }
 
-    public function get_supported_extensions() {
-        return $this->ext;
-    }
-
-    public function get_rank() {
-        return 10;
-    }
-}
-
-/**
- * Media renderer override for testing purposes.
- */
-class core_media_manager_test extends core_media_manager {
     /**
      * Access list of players as string, shortening it by getting rid of
      * repeated text.
+     * @param core_media_manager $manager The core_media_manager instance
      * @return string Comma-separated list of players
      */
-    public function get_players_test() {
-        $players = $this->get_players();
+    public function get_players_test($manager) {
+        $method = new \ReflectionMethod("core_media_manager", "get_players");
+        $method->setAccessible(true);
+        $players = $method->invoke($manager);
         $out = '';
         foreach ($players as $player) {
             if ($out) {
