@@ -23,9 +23,12 @@
  */
 
 namespace core_message;
+use DOMDocument;
 
 defined('MOODLE_INTERNAL') || die();
 
+require_once($CFG->dirroot . '/message/lib.php');
+
 /**
  * Helper class for the message area.
  *
@@ -35,81 +38,10 @@ defined('MOODLE_INTERNAL') || die();
 class helper {
 
     /**
-     * Helper function to retrieve the messages between two users
-     *
-     * TODO: This function should be removed once the new group messaging UI is in place and the old messaging UI is removed.
-     * For now we are not removing/deprecating this function for backwards compatibility with messaging UI.
-     * Followup: MDL-63915
-     *
-     * @param int $userid the current user
-     * @param int $otheruserid the other user
-     * @param int $timedeleted the time the message was deleted
-     * @param int $limitfrom
-     * @param int $limitnum
-     * @param string $sort
-     * @param int $timefrom the time from the message being sent
-     * @param int $timeto the time up until the message being sent
-     * @return array of messages
+     * @deprecated since 3.6
      */
-    public static function get_messages($userid, $otheruserid, $timedeleted = 0, $limitfrom = 0, $limitnum = 0,
-                                        $sort = 'timecreated ASC', $timefrom = 0, $timeto = 0) {
-        global $DB;
-
-        $hash = self::get_conversation_hash([$userid, $otheruserid]);
-
-        $sql = "SELECT m.id, m.useridfrom, m.subject, m.fullmessage, m.fullmessagehtml,
-                       m.fullmessageformat, m.smallmessage, m.timecreated, muaread.timecreated AS timeread
-                  FROM {message_conversations} mc
-            INNER JOIN {messages} m
-                    ON m.conversationid = mc.id
-             LEFT JOIN {message_user_actions} muaread
-                    ON (muaread.messageid = m.id
-                   AND muaread.userid = :userid1
-                   AND muaread.action = :readaction)";
-        $params = ['userid1' => $userid, 'readaction' => api::MESSAGE_ACTION_READ, 'convhash' => $hash];
-
-        if (empty($timedeleted)) {
-            $sql .= " LEFT JOIN {message_user_actions} mua
-                             ON (mua.messageid = m.id
-                            AND mua.userid = :userid2
-                            AND mua.action = :deleteaction
-                            AND mua.timecreated is NOT NULL)";
-        } else {
-            $sql .= " INNER JOIN {message_user_actions} mua
-                              ON (mua.messageid = m.id
-                             AND mua.userid = :userid2
-                             AND mua.action = :deleteaction
-                             AND mua.timecreated = :timedeleted)";
-            $params['timedeleted'] = $timedeleted;
-        }
-
-        $params['userid2'] = $userid;
-        $params['deleteaction'] = api::MESSAGE_ACTION_DELETED;
-
-        $sql .= " WHERE mc.convhash = :convhash";
-
-        if (!empty($timefrom)) {
-            $sql .= " AND m.timecreated >= :timefrom";
-            $params['timefrom'] = $timefrom;
-        }
-
-        if (!empty($timeto)) {
-            $sql .= " AND m.timecreated <= :timeto";
-            $params['timeto'] = $timeto;
-        }
-
-        if (empty($timedeleted)) {
-            $sql .= " AND mua.id is NULL";
-        }
-
-        $sql .= " ORDER BY m.$sort";
-
-        $messages = $DB->get_records_sql($sql, $params, $limitfrom, $limitnum);
-        foreach ($messages as &$message) {
-            $message->useridto = ($message->useridfrom == $userid) ? $otheruserid : $userid;
-        }
-
-        return $messages;
+    public static function get_messages() {
+        throw new \coding_exception('\core_message\helper::get_messages has been removed.');
     }
 
     /**
@@ -131,7 +63,8 @@ class helper {
         global $DB;
 
         $sql = "SELECT m.id, m.useridfrom, m.subject, m.fullmessage, m.fullmessagehtml,
-                       m.fullmessageformat, m.smallmessage, m.timecreated, muaread.timecreated AS timeread
+                       m.fullmessageformat, m.fullmessagetrust, m.smallmessage, m.timecreated,
+                       mc.contextid, muaread.timecreated AS timeread
                   FROM {message_conversations} mc
             INNER JOIN {messages} m
                     ON m.conversationid = mc.id
@@ -230,55 +163,10 @@ class helper {
     }
 
     /**
-     * Helper function to return an array of messages.
-     *
-     * TODO: This function should be removed once the new group messaging UI is in place and the old messaging UI is removed.
-     * For now we are not removing/deprecating this function for backwards compatibility with messaging UI.
-     * Followup: MDL-63915
-     *
-     * @param int $userid
-     * @param array $messages
-     * @return array
+     * @deprecated since 3.6
      */
-    public static function create_messages($userid, $messages) {
-        // Store the messages.
-        $arrmessages = array();
-
-        // We always view messages from oldest to newest, ensure we have it in that order.
-        $lastmessage = end($messages);
-        $firstmessage = reset($messages);
-        if ($lastmessage->timecreated < $firstmessage->timecreated) {
-            $messages = array_reverse($messages);
-        }
-
-        // Keeps track of the last day, month and year combo we were viewing.
-        $day = '';
-        $month = '';
-        $year = '';
-        foreach ($messages as $message) {
-            // Check if we are now viewing a different block period.
-            $displayblocktime = false;
-            $date = usergetdate($message->timecreated);
-            if ($day != $date['mday'] || $month != $date['month'] || $year != $date['year']) {
-                $day = $date['mday'];
-                $month = $date['month'];
-                $year = $date['year'];
-                $displayblocktime = true;
-            }
-            // Store the message to pass to the renderable.
-            $msg = new \stdClass();
-            $msg->id = $message->id;
-            $msg->text = message_format_message_text($message);
-            $msg->currentuserid = $userid;
-            $msg->useridfrom = $message->useridfrom;
-            $msg->useridto = $message->useridto;
-            $msg->displayblocktime = $displayblocktime;
-            $msg->timecreated = $message->timecreated;
-            $msg->timeread = $message->timeread;
-            $arrmessages[] = $msg;
-        }
-
-        return $arrmessages;
+    public static function create_messages() {
+        throw new \coding_exception('\core_message\helper::create_messages has been removed.');
     }
 
     /**
@@ -424,7 +312,9 @@ class helper {
      * @return array
      */
     public static function togglecontact_link_params($user, $iscontact = false) {
+        global $USER;
         $params = array(
+            'data-currentuserid' => $USER->id,
             'data-userid' => $user->id,
             'data-is-contact' => $iscontact,
             'id' => 'toggle-contact-button',
@@ -435,6 +325,40 @@ class helper {
         return $params;
     }
 
+    /**
+     * Requires the JS libraries for the message user button.
+     *
+     * @return void
+     */
+    public static function messageuser_requirejs() {
+        global $PAGE;
+
+        static $done = false;
+        if ($done) {
+            return;
+        }
+
+        $PAGE->requires->js_call_amd('core_message/message_user_button', 'send', array('#message-user-button'));
+        $done = true;
+    }
+
+    /**
+     * Returns the attributes to place on the message user button.
+     *
+     * @param int $useridto
+     * @return array
+     */
+    public static function messageuser_link_params(int $useridto) : array {
+        global $USER;
+
+        return [
+            'id' => 'message-user-button',
+            'role' => 'button',
+            'data-conversationid' => api::get_conversation_between_users([$USER->id, $useridto]),
+            'data-userid' => $useridto,
+        ];
+    }
+
     /**
      * Returns the conversation hash between users for easy look-ups in the DB.
      *
@@ -506,7 +430,8 @@ class helper {
         }
 
         list($useridsql, $usersparams) = $DB->get_in_or_equal($userids);
-        $userfields = \user_picture::fields('u', array('lastaccess'));
+        $userfieldsapi = \core_user\fields::for_userpic()->including('lastaccess');
+        $userfields = $userfieldsapi->get_sql('u', false, '', '', false)->selects;
         $userssql = "SELECT $userfields, u.deleted, mc.id AS contactid, mub.id AS blockedid
                        FROM {user} u
                   LEFT JOIN {message_contacts} mc
@@ -545,23 +470,24 @@ class helper {
 
             // Set contact and blocked status indicators.
             $data->iscontact = ($member->contactid) ? true : false;
-            $data->isblocked = ($member->blockedid) ? true : false;
+
+            // We don't want that a user has been blocked if they can message the user anyways.
+            $canmessageifblocked = api::can_send_message($referenceuserid, $member->id, true);
+            $data->isblocked = ($member->blockedid && !$canmessageifblocked) ? true : false;
 
             $data->isdeleted = ($member->deleted) ? true : false;
 
             $data->requirescontact = null;
             $data->canmessage = null;
+            $data->canmessageevenifblocked = null;
             if ($includeprivacyinfo) {
                 $privacysetting = api::get_user_privacy_messaging_preference($member->id);
                 $data->requirescontact = $privacysetting == api::MESSAGE_PRIVACY_ONLYCONTACTS;
 
-                $recipient = new \stdClass();
-                $recipient->id = $member->id;
-
-                $sender = new \stdClass();
-                $sender->id = $referenceuserid;
-
-                $data->canmessage = !$data->isdeleted && api::can_post_message($recipient, $sender);
+                // Here we check that if the sender wanted to block the recipient, the
+                // recipient would still be able to message them regardless.
+                $data->canmessageevenifblocked = !$data->isdeleted && $canmessageifblocked;
+                $data->canmessage = !$data->isdeleted && api::can_send_message($member->id, $referenceuserid);
             }
 
             // Populate the contact requests, even if we don't need them.
@@ -600,42 +526,186 @@ class helper {
 
         return $members;
     }
+    /**
+     * @deprecated since 3.6
+     */
+    public static function get_conversations_legacy_formatter() {
+        throw new \coding_exception('\core_message\helper::get_conversations_legacy_formatter has been removed.');
+    }
 
     /**
-     * Backwards compatibility formatter, transforming the new output of get_conversations() into the old format.
+     * Renders the messaging widget.
      *
-     * @param array $conversations the array of conversations, which must come from get_conversations().
-     * @return array the array of conversations, formatted in the legacy style.
+     * @param bool $isdrawer Are we are rendering the drawer or is this on a full page?
+     * @param int|null $sendtouser The ID of the user we want to send a message to
+     * @param int|null $conversationid The ID of the conversation we want to load
+     * @param string|null $view The first view to load in the message widget
+     * @return string The HTML.
      */
-    public static function get_conversations_legacy_formatter(array $conversations) : array {
-        // Transform new data format back into the old format, just for BC during the deprecation life cycle.
-        $tmp = [];
-        foreach ($conversations as $id => $conv) {
-            // Only individual conversations were supported in legacy messaging.
-            if ($conv->type != \core_message\api::MESSAGE_CONVERSATION_TYPE_INDIVIDUAL) {
-                continue;
-            }
-            $data = new \stdClass();
-            // The logic for the 'other user' is as follows:
-            // If a conversation is of type 'individual', the other user is always the member who is not the current user.
-            // If the conversation is of type 'group', the other user is always the sender of the most recent message.
-            // The get_conversations method already follows this logic, so we just need the first member.
-            $otheruser = reset($conv->members);
-            $data->userid = $otheruser->id;
-            $data->useridfrom = $conv->messages[0]->useridfrom ?? null;
-            $data->fullname = $conv->members[$otheruser->id]->fullname;
-            $data->profileimageurl = $conv->members[$otheruser->id]->profileimageurl;
-            $data->profileimageurlsmall = $conv->members[$otheruser->id]->profileimageurlsmall;
-            $data->ismessaging = isset($conv->messages[0]->text) ? true : false;
-            $data->lastmessage = $conv->messages[0]->text ? clean_param($conv->messages[0]->text, PARAM_NOTAGS) : null;
-            $data->lastmessagedate = $conv->messages[0]->timecreated ?? null;
-            $data->messageid = $conv->messages[0]->id ?? null;
-            $data->isonline = $conv->members[$otheruser->id]->isonline ?? null;
-            $data->isblocked = $conv->members[$otheruser->id]->isblocked ?? null;
-            $data->isread = $conv->isread;
-            $data->unreadcount = $conv->unreadcount;
-            $tmp[$data->userid] = $data;
+    public static function render_messaging_widget(
+        bool $isdrawer,
+        int $sendtouser = null,
+        int $conversationid = null,
+        string $view = null
+    ) {
+        global $USER, $CFG, $PAGE;
+
+        // Early bail out conditions.
+        if (empty($CFG->messaging) || !isloggedin() || isguestuser() || user_not_fully_set_up($USER) ||
+            get_user_preferences('auth_forcepasswordchange') ||
+            (!$USER->policyagreed && !is_siteadmin() &&
+                ($manager = new \core_privacy\local\sitepolicy\manager()) && $manager->is_defined())) {
+            return '';
         }
-        return $tmp;
+
+        $renderer = $PAGE->get_renderer('core');
+        $requestcount = \core_message\api::get_received_contact_requests_count($USER->id);
+        $contactscount = \core_message\api::count_contacts($USER->id);
+
+        $choices = [];
+        $choices[] = [
+            'value' => \core_message\api::MESSAGE_PRIVACY_ONLYCONTACTS,
+            'text' => get_string('contactableprivacy_onlycontacts', 'message')
+        ];
+        $choices[] = [
+            'value' => \core_message\api::MESSAGE_PRIVACY_COURSEMEMBER,
+            'text' => get_string('contactableprivacy_coursemember', 'message')
+        ];
+        if (!empty($CFG->messagingallusers)) {
+            // Add the MESSAGE_PRIVACY_SITE option when site-wide messaging between users is enabled.
+            $choices[] = [
+                'value' => \core_message\api::MESSAGE_PRIVACY_SITE,
+                'text' => get_string('contactableprivacy_site', 'message')
+            ];
+        }
+
+        // Enter to send.
+        $entertosend = get_user_preferences('message_entertosend', $CFG->messagingdefaultpressenter, $USER);
+
+        $notification = '';
+        if (!get_user_preferences('core_message_migrate_data', false)) {
+            $notification = get_string('messagingdatahasnotbeenmigrated', 'message');
+        }
+
+        if ($isdrawer) {
+            $template = 'core_message/message_drawer';
+            $messageurl = new \moodle_url('/message/index.php');
+        } else {
+            $template = 'core_message/message_index';
+            $messageurl = null;
+        }
+
+        $templatecontext = [
+            'contactrequestcount' => $requestcount,
+            'loggedinuser' => [
+                'id' => $USER->id,
+                'midnight' => usergetmidnight(time())
+            ],
+            // The starting timeout value for message polling.
+            'messagepollmin' => $CFG->messagingminpoll ?? MESSAGE_DEFAULT_MIN_POLL_IN_SECONDS,
+            // The maximum value that message polling timeout can reach.
+            'messagepollmax' => $CFG->messagingmaxpoll ?? MESSAGE_DEFAULT_MAX_POLL_IN_SECONDS,
+            // The timeout to reset back to after the max polling time has been reached.
+            'messagepollaftermax' => $CFG->messagingtimeoutpoll ?? MESSAGE_DEFAULT_TIMEOUT_POLL_IN_SECONDS,
+            'contacts' => [
+                'sectioncontacts' => [
+                    'placeholders' => array_fill(0, $contactscount > 50 ? 50 : $contactscount, true)
+                ],
+                'sectionrequests' => [
+                    'placeholders' => array_fill(0, $requestcount > 50 ? 50 : $requestcount, true)
+                ],
+            ],
+            'settings' => [
+                'privacy' => $choices,
+                'entertosend' => $entertosend
+            ],
+            'overview' => [
+                'messageurl' => $messageurl,
+                'notification' => $notification
+            ],
+            'isdrawer' => $isdrawer,
+            'showemojipicker' => !empty($CFG->allowemojipicker),
+            'messagemaxlength' => api::MESSAGE_MAX_LENGTH,
+            'caneditownmessageprofile' => has_capability('moodle/user:editownmessageprofile', \context_system::instance())
+        ];
+
+        if ($sendtouser || $conversationid) {
+            $route = [
+                'path' => 'view-conversation',
+                'params' => $conversationid ? [$conversationid] : [null, 'create', $sendtouser]
+            ];
+        } else if ($view === 'contactrequests') {
+            $route = [
+                'path' => 'view-contacts',
+                'params' => ['requests']
+            ];
+        } else {
+            $route = null;
+        }
+
+        $templatecontext['route'] = json_encode($route);
+
+        return $renderer->render_from_template($template, $templatecontext);
+    }
+
+    /**
+     * Returns user details for a user, if they are visible to the current user in the message search.
+     *
+     * This method checks the visibility of a user specifically for the purpose of inclusion in the message search results.
+     * Visibility depends on the site-wide messaging setting 'messagingallusers':
+     * If enabled, visibility depends only on the core notion of visibility; a visible site or course profile.
+     * If disabled, visibility requires that the user be sharing a course with the searching user, and have a visible profile there.
+     * The current user is always returned.
+     *
+     * @param \stdClass $user
+     * @return array the array of userdetails, if visible, or an empty array otherwise.
+     */
+    public static function search_get_user_details(\stdClass $user) : array {
+        global $CFG, $USER;
+        require_once($CFG->dirroot . '/user/lib.php');
+
+        if ($CFG->messagingallusers || $user->id == $USER->id) {
+            return \user_get_user_details_courses($user) ?? []; // This checks visibility of site and course profiles.
+        } else {
+            // Messaging specific: user must share a course with the searching user AND have a visible profile there.
+            $sharedcourses = enrol_get_shared_courses($USER, $user);
+            foreach ($sharedcourses as $course) {
+                if (user_can_view_profile($user, $course)) {
+                    $userdetails = user_get_user_details($user, $course);
+                    if (!is_null($userdetails)) {
+                        return $userdetails;
+                    }
+                }
+            }
+        }
+        return [];
+    }
+
+    /**
+     * Prevent unclosed HTML elements in a message.
+     *
+     * @param string $message The html message.
+     * @param bool $removebody True if we want to remove tag body.
+     * @return string The html properly structured.
+     */
+    public static function prevent_unclosed_html_tags(
+        string $message,
+        bool $removebody = false
+    ) : string {
+        $html = '';
+        if (!empty($message)) {
+            $doc = new DOMDocument();
+            $olderror = libxml_use_internal_errors(true);
+            $doc->loadHTML('<?xml version="1.0" encoding="UTF-8" ?>' . $message);
+            libxml_clear_errors();
+            libxml_use_internal_errors($olderror);
+            $html = $doc->getElementsByTagName('body')->item(0)->C14N(false, true);
+            if ($removebody) {
+                // Remove <body> element added in C14N function.
+                $html = preg_replace('~<(/?(?:body))[^>]*>\s*~i', '', $html);
+            }
+        }
+
+        return $html;
     }
 }
