@@ -24,7 +24,7 @@
  */
 
 require_once('../../config.php');
-require_once('lib.php');
+require_once('locallib.php');
 require_once("$CFG->libdir/rsslib.php");
 require_once("$CFG->libdir/form/filemanager.php");
 
@@ -33,9 +33,14 @@ $d     = optional_param('d', 0, PARAM_INT);    // database id
 $rid   = optional_param('rid', 0, PARAM_INT);    //record id
 $cancel   = optional_param('cancel', '', PARAM_RAW);    // cancel an add
 $mode ='addtemplate';    //define the mode for this page, only 1 mode available
+$tags = optional_param_array('tags', [], PARAM_TAGLIST);
 
 $url = new moodle_url('/mod/data/edit.php');
 if ($rid !== 0) {
+    $record = $DB->get_record('data_records', array(
+            'id' => $rid,
+            'dataid' => $d,
+        ), '*', MUST_EXIST);
     $url->param('rid', $rid);
 }
 if ($cancel !== '') {
@@ -106,7 +111,7 @@ $groupmode = groups_get_activity_groupmode($cm);
 if (!has_capability('mod/data:manageentries', $context)) {
     if ($rid) {
         // User is editing an existing record
-        if (!data_isowner($rid) || data_in_readonly_period($data)) {
+        if (!data_user_can_manage_entry($record, $data, $context)) {
             print_error('noaccess','data');
         }
     } else if (!data_user_can_add_entry($data, $currentgroup, $groupmode, $context)) {
@@ -151,113 +156,89 @@ if ($rid) {
 
 $PAGE->set_title($data->name);
 $PAGE->set_heading($course->fullname);
+$PAGE->force_settings_menu(true);
 
-/// Process incoming data for adding/updating records
+// Process incoming data for adding/updating records.
 
+// Keep track of any notifications.
+$generalnotifications = array();
+$fieldnotifications = array();
+
+// Process the submitted form.
 if ($datarecord = data_submitted() and confirm_sesskey()) {
+    if ($rid) {
+        // Updating an existing record.
 
-    $ignorenames = array('MAX_FILE_SIZE','sesskey','d','rid','saveandview','cancel');  // strings to be ignored in input data
+        // Retrieve the format for the fields.
+        $fields = $DB->get_records('data_fields', array('dataid' => $datarecord->d));
 
-    if ($rid) {                                          /// Update some records
+        // Validate the form to ensure that enough data was submitted.
+        $processeddata = data_process_submission($data, $fields, $datarecord);
 
-        /// All student edits are marked unapproved by default
-        $record = $DB->get_record('data_records', array('id'=>$rid));
+        // Add the new notification data.
+        $generalnotifications = array_merge($generalnotifications, $processeddata->generalnotifications);
+        $fieldnotifications = array_merge($fieldnotifications, $processeddata->fieldnotifications);
 
-        /// reset approved flag after student edit
-        if (!has_capability('mod/data:approve', $context)) {
-            $record->approved = 0;
+        if ($processeddata->validated) {
+            // Enough data to update the record.
+            data_update_record_fields_contents($data, $record, $context, $datarecord, $processeddata);
+            core_tag_tag::set_item_tags('mod_data', 'data_records', $rid, $context, $tags);
+
+            $viewurl = new moodle_url('/mod/data/view.php', array(
+                'd' => $data->id,
+                'rid' => $rid,
+            ));
+            redirect($viewurl);
         }
 
-        $record->groupid = $currentgroup;
-        $record->timemodified = time();
-        $DB->update_record('data_records', $record);
+    } else {
+        // No recordid was specified - creating a new entry.
 
-        /// Update all content
-        $field = NULL;
-        foreach ($datarecord as $name => $value) {
-            if (!in_array($name, $ignorenames)) {
-                $namearr = explode('_',$name);  // Second one is the field id
-                if (empty($field->field) || ($namearr[1] != $field->field->id)) {  // Try to reuse classes
-                    $field = data_get_field_from_id($namearr[1], $data);
-                }
-                if ($field) {
-                    $field->update_content($rid, $value, $name);
-                }
-            }
-        }
+        // Retrieve the format for the fields.
+        $fields = $DB->get_records('data_fields', array('dataid' => $datarecord->d));
 
-        // Trigger an event for updating this record.
-        $event = \mod_data\event\record_updated::create(array(
-            'objectid' => $rid,
-            'context' => $context,
-            'courseid' => $course->id,
-            'other' => array(
-                'dataid' => $data->id
-            )
-        ));
-        $event->add_record_snapshot('data', $data);
-        $event->trigger();
+        // Validate the form to ensure that enough data was submitted.
+        $processeddata = data_process_submission($data, $fields, $datarecord);
 
-        redirect($CFG->wwwroot.'/mod/data/view.php?d='.$data->id.'&rid='.$rid);
+        // Add the new notification data.
+        $generalnotifications = array_merge($generalnotifications, $processeddata->generalnotifications);
+        $fieldnotifications = array_merge($fieldnotifications, $processeddata->fieldnotifications);
 
-    } else { /// Add some new records
-        ///Empty form checking - you can't submit an empty form!
+        // Add instance to data_record.
+        if ($processeddata->validated && $recordid = data_add_record($data, $currentgroup)) {
 
-        $emptyform = true;      // assume the worst
+            // Now populate the fields contents of the new record.
+            data_add_fields_contents_to_new_record($data, $context, $recordid, $fields, $datarecord, $processeddata);
 
-        foreach ($datarecord as $name => $value) {
-            if (!in_array($name, $ignorenames)) {
-                $namearr = explode('_', $name);  // Second one is the field id
-                if (empty($field->field) || ($namearr[1] != $field->field->id)) {  // Try to reuse classes
-                    $field = data_get_field_from_id($namearr[1], $data);
-                }
-                if ($field->notemptyfield($value, $name)) {
-                    $emptyform = false;
-                    break;             // if anything has content, this form is not empty, so stop now!
-                }
-            }
-        }
-
-        if ($emptyform){    //nothing gets written to database
-            echo $OUTPUT->notification(get_string('emptyaddform','data'));
-        }
-
-        if (!$emptyform && $recordid = data_add_record($data, $currentgroup)) {    //add instance to data_record
-
-            /// Insert a whole lot of empty records to make sure we have them
-            $fields = $DB->get_records('data_fields', array('dataid'=>$data->id));
-            foreach ($fields as $field) {
-                $content = new stdClass();
-                $content->recordid = $recordid;
-                $content->fieldid = $field->id;
-                $DB->insert_record('data_content',$content);
-            }
-
-            /// For each field in the add form, add it to the data_content.
-            foreach ($datarecord as $name => $value){
-                if (!in_array($name, $ignorenames)) {
-                    $namearr = explode('_', $name);  // Second one is the field id
-                    if (empty($field->field) || ($namearr[1] != $field->field->id)) {  // Try to reuse classes
-                        $field = data_get_field_from_id($namearr[1], $data);
-                    }
-                    if ($field) {
-                        $field->update_content($recordid, $value, $name);
-                    }
-                }
-            }
+            core_tag_tag::set_item_tags('mod_data', 'data_records', $recordid, $context, $tags);
 
             if (!empty($datarecord->saveandview)) {
-                redirect($CFG->wwwroot.'/mod/data/view.php?d='.$data->id.'&rid='.$recordid);
+                $viewurl = new moodle_url('/mod/data/view.php', array(
+                    'd' => $data->id,
+                    'rid' => $recordid,
+                ));
+                redirect($viewurl);
+            } else if (!empty($datarecord->saveandadd)) {
+                // User has clicked "Save and add another". Reset all of the fields.
+                $datarecord = null;
             }
         }
     }
-}  // End of form processing
+}
+// End of form processing.
 
 
 /// Print the page header
 
 echo $OUTPUT->header();
 echo $OUTPUT->heading(format_string($data->name), 2);
+
+// Render the activity information.
+$cminfo = cm_info::create($cm);
+$completiondetails = \core_completion\cm_completion_details::get_instance($cminfo, $USER->id);
+$activitydates = \core\activity_dates::get_dates_for_module($cminfo, $USER->id);
+echo $OUTPUT->activity_information($cminfo, $completiondetails, $activitydates);
+
 echo $OUTPUT->box(format_module_intro('data', $data, $cm->id), 'generalbox', 'intro');
 groups_print_activity_menu($cm, $CFG->wwwroot.'/mod/data/edit.php?d='.$data->id);
 
@@ -296,13 +277,43 @@ if ($data->addtemplate){
     $replacements = array();
 
     ///then we generate strings to replace
-    foreach ($possiblefields as $eachfield){
+    foreach ($possiblefields as $eachfield) {
         $field = data_get_field($eachfield, $data);
-        $patterns[]="[[".$field->field->name."]]";
-        $replacements[] = $field->display_add_field($rid);
-        $patterns[]="[[".$field->field->name."#id]]";
-        $replacements[] = 'field_'.$field->field->id;
+
+        // To skip unnecessary calls to display_add_field().
+        if (strpos($data->addtemplate, "[[" . $field->field->name . "]]") !== false) {
+            // Display an error in case the field type is not found.
+            $errors = '';
+            if (!empty($fieldnotifications[$field->field->name])) {
+                foreach ($fieldnotifications[$field->field->name] as $notification) {
+                    $errors .= $OUTPUT->notification($notification);
+                }
+            }
+
+            // Replace the field tag.
+            $fielddisplay = '';
+            if ($field->type === 'unknown') {
+                if (has_capability('mod/data:manageentries', $context)) {
+                    // Display notification for users that can manage entries.
+                    $errors .= $OUTPUT->notification(get_string('missingfieldtype', 'data',
+                    (object)['name' => $field->field->name]));
+                }
+            } else {
+                $fielddisplay = $field->display_add_field($rid, $datarecord);
+            }
+
+            $patterns[] = "[[" . $field->field->name . "]]";
+            $replacements[] = $errors . $fielddisplay;
+        }    // Replace the field id tag.
+        $patterns[] = "[[" . $field->field->name . "#id]]";
+        $replacements[] = 'field_' . $field->field->id;
     }
+
+    if (core_tag_tag::is_enabled('mod_data', 'data_records')) {
+        $patterns[] = "##tags##";
+        $replacements[] = data_generate_tag_form($rid);
+    }
+
     $newtext = str_ireplace($patterns, $replacements, $data->{$mode});
 
 } else {    //if the add template is not yet defined, print the default form!
@@ -310,14 +321,22 @@ if ($data->addtemplate){
     $newtext = '';
 }
 
+foreach ($generalnotifications as $notification) {
+    echo $OUTPUT->notification($notification);
+}
 echo $newtext;
 
-echo '<div class="mdl-align"><input type="submit" name="saveandview" value="'.get_string('saveandview','data').'" />';
+echo '<div class="mdl-align mt-1"><input type="submit" class="btn btn-primary" name="saveandview" ' .
+     'value="' . get_string('saveandview', 'data') . '" />';
 if ($rid) {
-    echo '&nbsp;<input type="submit" name="cancel" value="'.get_string('cancel').'" onclick="javascript:history.go(-1)" />';
+    echo '&nbsp;<input type="submit" class="btn btn-primary" name="cancel" ' .
+         'value="' . get_string('cancel') . '" onclick="javascript:history.go(-1)" />';
 } else {
-    if ((!$data->maxentries) || has_capability('mod/data:manageentries', $context) || (data_numentries($data) < ($data->maxentries - 1))) {
-        echo '&nbsp;<input type="submit" value="'.get_string('saveandadd','data').'" />';
+    if ((!$data->maxentries) ||
+            has_capability('mod/data:manageentries', $context) ||
+            (data_numentries($data) < ($data->maxentries - 1))) {
+        echo '&nbsp;<input type="submit" class="btn btn-primary" name="saveandadd" ' .
+             'value="' . get_string('saveandadd', 'data') . '" />';
     }
 }
 echo '</div>';
