@@ -543,8 +543,9 @@ class summary_table extends table_sql {
     protected function define_base_sql(): void {
         global $USER;
 
-        $userfields = get_extra_user_fields($this->userfieldscontext);
-        $userfieldssql = \user_picture::fields('u', $userfields);
+        // TODO Does not support custom user profile fields (MDL-70456).
+        $userfieldsapi = \core_user\fields::for_identity($this->userfieldscontext, false)->with_userpic();
+        $userfieldssql = $userfieldsapi->get_sql('u', false, '', '', false)->selects;
 
         // Define base SQL query format.
         $this->sql->basefields = ' u.id AS userid,
@@ -585,26 +586,21 @@ class summary_table extends table_sql {
         [$enrolleduserssql, $enrolledusersparams] = get_enrolled_sql($this->get_context(), '', $groupids);
         $this->sql->params += $enrolledusersparams;
 
-        $this->sql->basefromjoins = '    {user} u
-                                    JOIN (' . $enrolleduserssql . ') enrolledusers ON enrolledusers.id = u.id
-                                    JOIN {forum} f ON f.course = :forumcourseid
-                                    JOIN {forum_discussions} d ON d.forum = f.id
-                               LEFT JOIN {forum_posts} p ON p.discussion =  d.id
-                                     AND p.userid = u.id
-                                     ' . $privaterepliessql
-                                       . $this->sql->filterbase['dates'] . '
-                               LEFT JOIN (
-                                            SELECT COUNT(fi.id) AS attcount, fi.itemid AS postid, fi.userid
-                                              FROM {files} fi
-                                             WHERE fi.component = :component
-                                               AND fi.filesize > 0
-                                          GROUP BY fi.itemid, fi.userid
-                                         ) att ON att.postid = p.id
-                                         AND att.userid = u.id';
+        $queryattachments = 'SELECT COUNT(fi.id) AS attcount, fi.itemid AS postid, fi.userid
+                               FROM {files} fi
+                              WHERE fi.component = :component AND fi.filesize > 0
+                           GROUP BY fi.itemid, fi.userid';
+        $this->sql->basefromjoins = ' {user} u
+                                 JOIN (' . $enrolleduserssql . ') enrolledusers ON enrolledusers.id = u.id
+                                 JOIN {forum} f ON f.course = :forumcourseid
+                                 JOIN {forum_discussions} d ON d.forum = f.id
+                            LEFT JOIN {forum_posts} p ON p.discussion = d.id AND p.userid = u.id '
+                                    . $privaterepliessql
+                                    . $this->sql->filterbase['dates'] . '
+                            LEFT JOIN (' . $queryattachments . ') att ON att.postid = p.id AND att.userid = u.id';
 
-        $this->sql->basewhere = '1=1';
-
-        $this->sql->basegroupby = $userfieldssql . ', d.course';
+        $this->sql->basewhere = '1 = 1';
+        $this->sql->basegroupby = "$userfieldssql, d.course";
 
         if ($this->logreader) {
             $this->fill_log_summary_temp_table();
@@ -736,7 +732,7 @@ class summary_table extends table_sql {
 
             $groupby = ' GROUP BY ' . $this->sql->basegroupby . $this->sql->filtergroupby;
 
-            if (($sort = $this->get_sql_sort())) {
+            if ($sort = $this->get_sql_sort()) {
                 $orderby = " ORDER BY {$sort}";
             }
         } else {
