@@ -43,7 +43,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for get_quizzes_by_courses.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function get_quizzes_by_courses_parameters() {
@@ -105,12 +105,17 @@ class mod_quiz_external extends external_api {
 
                 if (has_capability('mod/quiz:view', $context)) {
                     // Format intro.
-                    list($quizdetails['intro'], $quizdetails['introformat']) = external_format_text($quiz->intro,
-                                                                    $quiz->introformat, $context->id, 'mod_quiz', 'intro', null);
+                    $options = array('noclean' => true);
+                    list($quizdetails['intro'], $quizdetails['introformat']) =
+                        external_format_text($quiz->intro, $quiz->introformat, $context->id, 'mod_quiz', 'intro', null, $options);
 
                     $quizdetails['introfiles'] = external_util::get_area_files($context->id, 'mod_quiz', 'intro', false, false);
                     $viewablefields = array('timeopen', 'timeclose', 'grademethod', 'section', 'visible', 'groupmode',
-                                            'groupingid');
+                                            'groupingid', 'attempts', 'timelimit', 'grademethod', 'decimalpoints',
+                                            'questiondecimalpoints', 'sumgrades', 'grade', 'preferredbehaviour');
+                    // Some times this function returns just empty.
+                    $hasfeedback = quiz_has_feedback($quiz);
+                    $quizdetails['hasfeedback'] = (!empty($hasfeedback)) ? 1 : 0;
 
                     $timenow = time();
                     $quizobj = quiz::create($quiz->id, $USER->id);
@@ -119,20 +124,15 @@ class mod_quiz_external extends external_api {
 
                     // Fields the user could see if have access to the quiz.
                     if (!$accessmanager->prevent_access()) {
-                        // Some times this function returns just empty.
-                        $hasfeedback = quiz_has_feedback($quiz);
-                        $quizdetails['hasfeedback'] = (!empty($hasfeedback)) ? 1 : 0;
                         $quizdetails['hasquestions'] = (int) $quizobj->has_questions();
                         $quizdetails['autosaveperiod'] = get_config('quiz', 'autosaveperiod');
 
-                        $additionalfields = array('timelimit', 'attempts', 'attemptonlast', 'grademethod', 'decimalpoints',
-                                                    'questiondecimalpoints', 'reviewattempt', 'reviewcorrectness', 'reviewmarks',
+                        $additionalfields = array('attemptonlast', 'reviewattempt', 'reviewcorrectness', 'reviewmarks',
                                                     'reviewspecificfeedback', 'reviewgeneralfeedback', 'reviewrightanswer',
-                                                    'reviewoverallfeedback', 'questionsperpage', 'navmethod', 'sumgrades', 'grade',
+                                                    'reviewoverallfeedback', 'questionsperpage', 'navmethod',
                                                     'browsersecurity', 'delay1', 'delay2', 'showuserpicture', 'showblocks',
                                                     'completionattemptsexhausted', 'completionpass', 'overduehandling',
-                                                    'graceperiod', 'preferredbehaviour', 'canredoquestions',
-                                                    'allowofflineattempts');
+                                                    'graceperiod', 'canredoquestions', 'allowofflineattempts');
                         $viewablefields = array_merge($viewablefields, $additionalfields);
                     }
 
@@ -301,7 +301,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for view_quiz.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function view_quiz_parameters() {
@@ -355,7 +355,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for get_user_attempts.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function get_user_attempts_parameters() {
@@ -413,10 +413,21 @@ class mod_quiz_external extends external_api {
             require_capability('mod/quiz:viewreports', $context);
         }
 
+        // Update quiz with override information.
+        $quiz = quiz_update_effective_access($quiz, $params['userid']);
         $attempts = quiz_get_user_attempts($quiz->id, $user->id, $params['status'], $params['includepreviews']);
-
+        $attemptresponse = [];
+        foreach ($attempts as $attempt) {
+            $reviewoptions = quiz_get_review_options($quiz, $attempt, $context);
+            if (!has_capability('mod/quiz:viewreports', $context) &&
+                    ($reviewoptions->marks < question_display_options::MARK_AND_MAX || $attempt->state != quiz_attempt::FINISHED)) {
+                // Blank the mark if the teacher does not allow it.
+                $attempt->sumgrades = null;
+            }
+            $attemptresponse[] = $attempt;
+        }
         $result = array();
-        $result['attempts'] = $attempts;
+        $result['attempts'] = $attemptresponse;
         $result['warnings'] = $warnings;
         return $result;
     }
@@ -474,7 +485,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for get_user_best_grade.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function get_user_best_grade_parameters() {
@@ -495,7 +506,8 @@ class mod_quiz_external extends external_api {
      * @since Moodle 3.1
      */
     public static function get_user_best_grade($quizid, $userid = 0) {
-        global $DB, $USER;
+        global $DB, $USER, $CFG;
+        require_once($CFG->libdir . '/gradelib.php');
 
         $warnings = array();
 
@@ -521,7 +533,23 @@ class mod_quiz_external extends external_api {
         }
 
         $result = array();
-        $grade = quiz_get_best_grade($quiz, $user->id);
+
+        // This code was mostly copied from mod/quiz/view.php. We need to make the web service logic consistent.
+        // Get this user's attempts.
+        $attempts = quiz_get_user_attempts($quiz->id, $user->id, 'all');
+        $canviewgrade = false;
+        if ($attempts) {
+            if ($USER->id != $user->id) {
+                // No need to check the permission here. We did it at by require_capability('mod/quiz:viewreports', $context).
+                $canviewgrade = true;
+            } else {
+                // Work out which columns we need, taking account what data is available in each attempt.
+                [$notused, $alloptions] = quiz_get_combined_reviewoptions($quiz, $attempts);
+                $canviewgrade = $alloptions->marks >= question_display_options::MARK_AND_MAX;
+            }
+        }
+
+        $grade = $canviewgrade ? quiz_get_best_grade($quiz, $user->id) : null;
 
         if ($grade === null) {
             $result['hasgrade'] = false;
@@ -529,6 +557,17 @@ class mod_quiz_external extends external_api {
             $result['hasgrade'] = true;
             $result['grade'] = $grade;
         }
+
+        // Inform user of the grade to pass if non-zero.
+        $gradinginfo = grade_get_grades($course->id, 'mod', 'quiz', $quiz->id, $user->id);
+        if (!empty($gradinginfo->items)) {
+            $item = $gradinginfo->items[0];
+
+            if ($item && grade_floats_different($item->gradepass, 0)) {
+                $result['gradetopass'] = $item->gradepass;
+            }
+        }
+
         $result['warnings'] = $warnings;
         return $result;
     }
@@ -544,6 +583,7 @@ class mod_quiz_external extends external_api {
             array(
                 'hasgrade' => new external_value(PARAM_BOOL, 'Whether the user has a grade on the given quiz.'),
                 'grade' => new external_value(PARAM_FLOAT, 'The grade (only if the user has a grade).', VALUE_OPTIONAL),
+                'gradetopass' => new external_value(PARAM_FLOAT, 'The grade to pass the quiz (only if set).', VALUE_OPTIONAL),
                 'warnings' => new external_warnings(),
             )
         );
@@ -552,7 +592,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for get_combined_review_options.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function get_combined_review_options_parameters() {
@@ -653,7 +693,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for start_attempt.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function start_attempt_parameters() {
@@ -866,7 +906,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes a single question structure.
      *
-     * @return external_single_structure the question structure
+     * @return external_single_structure the question data. Some fields may not be returned depending on the quiz display settings.
      * @since  Moodle 3.1
      * @since Moodle 3.2 blockedbyprevious parameter added.
      */
@@ -877,6 +917,14 @@ class mod_quiz_external extends external_api {
                 'type' => new external_value(PARAM_ALPHANUMEXT, 'question type, i.e: multichoice'),
                 'page' => new external_value(PARAM_INT, 'page of the quiz this question appears on'),
                 'html' => new external_value(PARAM_RAW, 'the question rendered'),
+                'responsefileareas' => new external_multiple_structure(
+                    new external_single_structure(
+                        array(
+                            'area' => new external_value(PARAM_NOTAGS, 'File area name'),
+                            'files' => new external_files('Response files for the question', VALUE_OPTIONAL),
+                        )
+                    ), 'Response file areas including files', VALUE_OPTIONAL
+                ),
                 'sequencecheck' => new external_value(PARAM_INT, 'the number of real steps in this attempt', VALUE_OPTIONAL),
                 'lastactiontime' => new external_value(PARAM_INT, 'the timestamp of the most recent step in this question attempt',
                                                         VALUE_OPTIONAL),
@@ -884,13 +932,19 @@ class mod_quiz_external extends external_api {
                                                             VALUE_OPTIONAL),
                 'flagged' => new external_value(PARAM_BOOL, 'whether the question is flagged or not'),
                 'number' => new external_value(PARAM_INT, 'question ordering number in the quiz', VALUE_OPTIONAL),
-                'state' => new external_value(PARAM_ALPHA, 'the state where the question is in', VALUE_OPTIONAL),
+                'state' => new external_value(PARAM_ALPHA, 'the state where the question is in.
+                    It will not be returned if the user cannot see it due to the quiz display correctness settings.',
+                    VALUE_OPTIONAL),
                 'status' => new external_value(PARAM_RAW, 'current formatted state of the question', VALUE_OPTIONAL),
                 'blockedbyprevious' => new external_value(PARAM_BOOL, 'whether the question is blocked by the previous question',
-                        VALUE_OPTIONAL),
-                'mark' => new external_value(PARAM_RAW, 'the mark awarded', VALUE_OPTIONAL),
-                'maxmark' => new external_value(PARAM_FLOAT, 'the maximum mark possible for this question attempt', VALUE_OPTIONAL),
-            )
+                    VALUE_OPTIONAL),
+                'mark' => new external_value(PARAM_RAW, 'the mark awarded.
+                    It will be returned only if the user is allowed to see it.', VALUE_OPTIONAL),
+                'maxmark' => new external_value(PARAM_FLOAT, 'the maximum mark possible for this question attempt.
+                    It will be returned only if the user is allowed to see it.', VALUE_OPTIONAL),
+                'settings' => new external_value(PARAM_RAW, 'Question settings (JSON encoded).', VALUE_OPTIONAL),
+            ),
+            'The question data. Some fields may not be returned depending on the quiz display settings.'
         );
     }
 
@@ -909,34 +963,67 @@ class mod_quiz_external extends external_api {
         $contextid = $attemptobj->get_quizobj()->get_context()->id;
         $displayoptions = $attemptobj->get_display_options($review);
         $renderer = $PAGE->get_renderer('mod_quiz');
+        $contextid = $attemptobj->get_quizobj()->get_context()->id;
 
         foreach ($attemptobj->get_slots($page) as $slot) {
+            $qtype = $attemptobj->get_question_type_name($slot);
+            $qattempt = $attemptobj->get_question_attempt($slot);
+            $questiondef = $qattempt->get_question(true);
+
+            // Get response files (for questions like essay that allows attachments).
+            $responsefileareas = [];
+            foreach (question_bank::get_qtype($qtype)->response_file_areas() as $area) {
+                if ($files = $attemptobj->get_question_attempt($slot)->get_last_qt_files($area, $contextid)) {
+                    $responsefileareas[$area]['area'] = $area;
+                    $responsefileareas[$area]['files'] = [];
+
+                    foreach ($files as $file) {
+                        $responsefileareas[$area]['files'][] = array(
+                            'filename' => $file->get_filename(),
+                            'fileurl' => $qattempt->get_response_file_url($file),
+                            'filesize' => $file->get_filesize(),
+                            'filepath' => $file->get_filepath(),
+                            'mimetype' => $file->get_mimetype(),
+                            'timemodified' => $file->get_timemodified(),
+                        );
+                    }
+                }
+            }
+
+            // Check display settings for question.
+            $settings = $questiondef->get_question_definition_for_external_rendering($qattempt, $displayoptions);
 
             $question = array(
                 'slot' => $slot,
-                'type' => $attemptobj->get_question_type_name($slot),
+                'type' => $qtype,
                 'page' => $attemptobj->get_question_page($slot),
                 'flagged' => $attemptobj->is_question_flagged($slot),
                 'html' => $attemptobj->render_question($slot, $review, $renderer) . $PAGE->requires->get_end_code(),
-                'sequencecheck' => $attemptobj->get_question_attempt($slot)->get_sequence_check_count(),
-                'lastactiontime' => $attemptobj->get_question_attempt($slot)->get_last_step()->get_timecreated(),
-                'hasautosavedstep' => $attemptobj->get_question_attempt($slot)->has_autosaved_step()
+                'responsefileareas' => $responsefileareas,
+                'sequencecheck' => $qattempt->get_sequence_check_count(),
+                'lastactiontime' => $qattempt->get_last_step()->get_timecreated(),
+                'hasautosavedstep' => $qattempt->has_autosaved_step(),
+                'settings' => !empty($settings) ? json_encode($settings) : null,
             );
 
             if ($attemptobj->is_real_question($slot)) {
                 $question['number'] = $attemptobj->get_question_number($slot);
-                $question['state'] = (string) $attemptobj->get_question_state($slot);
+                $showcorrectness = $displayoptions->correctness && $qattempt->has_marks();
+                if ($showcorrectness) {
+                    $question['state'] = (string) $attemptobj->get_question_state($slot);
+                }
                 $question['status'] = $attemptobj->get_question_status($slot, $displayoptions->correctness);
                 $question['blockedbyprevious'] = $attemptobj->is_blocked_by_previous_question($slot);
             }
             if ($displayoptions->marks >= question_display_options::MAX_ONLY) {
-                $question['maxmark'] = $attemptobj->get_question_attempt($slot)->get_max_mark();
+                $question['maxmark'] = $qattempt->get_max_mark();
             }
             if ($displayoptions->marks >= question_display_options::MARK_AND_MAX) {
                 $question['mark'] = $attemptobj->get_question_mark($slot);
             }
-
-            $questions[] = $question;
+            if ($attemptobj->check_page_access($attemptobj->get_question_page($slot), false)) {
+                $questions[] = $question;
+            }
         }
         return $questions;
     }
@@ -944,7 +1031,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for get_attempt_data.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function get_attempt_data_parameters() {
@@ -1027,7 +1114,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for get_attempt_summary.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function get_attempt_summary_parameters() {
@@ -1091,7 +1178,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for save_attempt.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function save_attempt_parameters() {
@@ -1128,7 +1215,7 @@ class mod_quiz_external extends external_api {
      * @since Moodle 3.1
      */
     public static function save_attempt($attemptid, $data, $preflightdata = array()) {
-        global $DB;
+        global $DB, $USER;
 
         $warnings = array();
 
@@ -1142,11 +1229,17 @@ class mod_quiz_external extends external_api {
         // Add a page, required by validate_attempt.
         list($attemptobj, $messages) = self::validate_attempt($params);
 
+        // Prevent functions like file_get_submitted_draft_itemid() or form library requiring a sesskey for WS requests.
+        if (WS_SERVER || PHPUNIT_TEST) {
+            $USER->ignoresesskey = true;
+        }
         $transaction = $DB->start_delegated_transaction();
         // Create the $_POST object required by the question engine.
         $_POST = array();
         foreach ($data as $element) {
             $_POST[$element['name']] = $element['value'];
+            // Some deep core functions like file_get_submitted_draft_itemid() also requires $_REQUEST to be filled.
+            $_REQUEST[$element['name']] = $element['value'];
         }
         $timenow = time();
         // Update the timemodifiedoffline field.
@@ -1178,7 +1271,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for process_attempt.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function process_attempt_parameters() {
@@ -1221,6 +1314,7 @@ class mod_quiz_external extends external_api {
      * @since Moodle 3.1
      */
     public static function process_attempt($attemptid, $data, $finishattempt = false, $timeup = false, $preflightdata = array()) {
+        global $USER;
 
         $warnings = array();
 
@@ -1233,13 +1327,21 @@ class mod_quiz_external extends external_api {
         );
         $params = self::validate_parameters(self::process_attempt_parameters(), $params);
 
-        // Do not check access manager rules.
-        list($attemptobj, $messages) = self::validate_attempt($params, false);
+        // Do not check access manager rules and evaluate fail if overdue.
+        $attemptobj = quiz_attempt::create($params['attemptid']);
+        $failifoverdue = !($attemptobj->get_quizobj()->get_quiz()->overduehandling == 'graceperiod');
 
+        list($attemptobj, $messages) = self::validate_attempt($params, false, $failifoverdue);
+
+        // Prevent functions like file_get_submitted_draft_itemid() or form library requiring a sesskey for WS requests.
+        if (WS_SERVER || PHPUNIT_TEST) {
+            $USER->ignoresesskey = true;
+        }
         // Create the $_POST object required by the question engine.
         $_POST = array();
         foreach ($params['data'] as $element) {
             $_POST[$element['name']] = $element['value'];
+            $_REQUEST[$element['name']] = $element['value'];
         }
         $timenow = time();
         $finishattempt = $params['finishattempt'];
@@ -1289,7 +1391,8 @@ class mod_quiz_external extends external_api {
             if (!$attemptobj->is_finished()) {
                 throw new moodle_quiz_exception($attemptobj->get_quizobj(), 'attemptclosed');
             } else if (!$displayoptions->attempt) {
-                throw new moodle_exception($attemptobj->cannot_review_message());
+                throw new moodle_quiz_exception($attemptobj->get_quizobj(), 'noreview', null, '',
+                    $attemptobj->cannot_review_message());
             }
         } else if (!$attemptobj->is_review_allowed()) {
             throw new moodle_quiz_exception($attemptobj->get_quizobj(), 'noreviewattempt');
@@ -1300,7 +1403,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for get_attempt_review.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function get_attempt_review_parameters() {
@@ -1405,7 +1508,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for view_attempt.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function view_attempt_parameters() {
@@ -1478,7 +1581,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for view_attempt_summary.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function view_attempt_summary_parameters() {
@@ -1543,7 +1646,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for view_attempt_review.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function view_attempt_review_parameters() {
@@ -1598,7 +1701,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for view_quiz.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function get_quiz_feedback_for_grade_parameters() {
@@ -1671,7 +1774,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for get_quiz_access_information.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function get_quiz_access_information_parameters() {
@@ -1753,7 +1856,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for get_attempt_access_information.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function get_attempt_access_information_parameters() {
@@ -1860,7 +1963,7 @@ class mod_quiz_external extends external_api {
     /**
      * Describes the parameters for get_quiz_required_qtypes.
      *
-     * @return external_external_function_parameters
+     * @return external_function_parameters
      * @since Moodle 3.1
      */
     public static function get_quiz_required_qtypes_parameters() {
