@@ -154,7 +154,7 @@ function quiz_update_instance($quiz, $mform) {
     quiz_delete_previews($quiz);
 
     // Repaginate, if asked to.
-    if (!empty($quiz->repaginatenow)) {
+    if (!empty($quiz->repaginatenow) && !quiz_has_attempts($quiz->id)) {
         quiz_repaginate_questions($quiz->id, $quiz->questionsperpage);
     }
 
@@ -1128,14 +1128,17 @@ function quiz_process_options($quiz) {
     $quiz->reviewoverallfeedback &= ~mod_quiz_display_options::DURING;
 
     // Ensure that disabled checkboxes in completion settings are set to 0.
-    if (empty($quiz->completionusegrade)) {
-        $quiz->completionpass = 0;
-    }
-    if (empty($quiz->completionpass)) {
-        $quiz->completionattemptsexhausted = 0;
-    }
-    if (empty($quiz->completionminattemptsenabled)) {
-        $quiz->completionminattempts = 0;
+    // But only if the completion settinsg are unlocked.
+    if (!empty($quiz->completionunlocked)) {
+        if (empty($quiz->completionusegrade)) {
+            $quiz->completionpass = 0;
+        }
+        if (empty($quiz->completionpass)) {
+            $quiz->completionattemptsexhausted = 0;
+        }
+        if (empty($quiz->completionminattemptsenabled)) {
+            $quiz->completionminattempts = 0;
+        }
     }
 }
 
