@@ -247,8 +247,8 @@ class behat_mod_quiz extends behat_question_base {
             if (!array_key_exists('maxmark', $questiondata) || $questiondata['maxmark'] === '') {
                 $maxmark = null;
             } else {
-                $maxmark = clean_param($questiondata['maxmark'], PARAM_FLOAT);
-                if (!is_numeric($questiondata['maxmark']) || $maxmark < 0) {
+                $maxmark = clean_param($questiondata['maxmark'], PARAM_LOCALISEDFLOAT);
+                if (!is_numeric($maxmark) || $maxmark < 0) {
                     throw new ExpectationException('The max mark for question "' .
                             $questiondata['question'] . '" must be a positive number.',
                             $this->getSession());
