@@ -83,7 +83,7 @@ function url_fix_submitted_url($url) {
  */
 function url_get_full_url($url, $cm, $course, $config=null) {
 
-    $parameters = empty($url->parameters) ? array() : unserialize($url->parameters);
+    $parameters = empty($url->parameters) ? [] : (array) unserialize_array($url->parameters);
 
     // make sure there are no encoded entities, it is ok to do this twice
     $fullurl = html_entity_decode($url->externalurl, ENT_QUOTES, 'UTF-8');
@@ -195,7 +195,7 @@ function url_print_heading($url, $cm, $course, $notused = false) {
 function url_print_intro($url, $cm, $course, $ignoresettings=false) {
     global $OUTPUT;
 
-    $options = empty($url->displayoptions) ? array() : unserialize($url->displayoptions);
+    $options = empty($url->displayoptions) ? [] : (array) unserialize_array($url->displayoptions);
     if ($ignoresettings or !empty($options['printintro'])) {
         if (trim(strip_tags($url->intro))) {
             echo $OUTPUT->box_start('mod_introbox', 'urlintro');
@@ -284,7 +284,7 @@ function url_print_workaround($url, $cm, $course) {
     $display = url_get_final_display_type($url);
     if ($display == RESOURCELIB_DISPLAY_POPUP) {
         $jsfullurl = addslashes_js($fullurl);
-        $options = empty($url->displayoptions) ? array() : unserialize($url->displayoptions);
+        $options = empty($url->displayoptions) ? [] : (array) unserialize_array($url->displayoptions);
         $width  = empty($options['popupwidth'])  ? 620 : $options['popupwidth'];
         $height = empty($options['popupheight']) ? 450 : $options['popupheight'];
         $wh = "width=$width,height=$height,toolbar=no,location=no,menubar=no,copyhistory=no,status=no,directories=no,scrollbars=yes,resizable=yes";
@@ -380,8 +380,9 @@ function url_get_final_display_type($url) {
         }
     }
 
-    static $download = array('application/zip', 'application/x-tar', 'application/g-zip',     // binary formats
-                             'application/pdf', 'text/html');  // these are known to cause trouble for external links, sorry
+    // Binaries and other formats that are known to cause trouble for external links.
+    static $download = ['application/zip', 'application/x-tar', 'application/g-zip',
+                        'application/pdf', 'text/html', 'document/unknown'];
     static $embed    = array('image/gif', 'image/jpeg', 'image/png', 'image/svg+xml',         // images
                              'application/x-shockwave-flash', 'video/x-flv', 'video/x-ms-wm', // video formats
                              'video/quicktime', 'video/mpeg', 'video/mp4',
@@ -570,12 +571,23 @@ function url_guess_icon($fullurl, $size = null) {
         return null;
     }
 
+    try {
+        // There can be some cases where the url is invalid making parse_url() to return false.
+        // That will make moodle_url class to throw an exception, so we need to catch the exception to prevent errors.
+        $moodleurl = new moodle_url($fullurl);
+        $fullurl = $moodleurl->out_omit_querystring();
+    } catch (\moodle_exception $e) {
+        // If an exception is thrown, means the url is invalid. No need to log exception.
+        return null;
+    }
+
     $icon = file_extension_icon($fullurl, $size);
     $htmlicon = file_extension_icon('.htm', $size);
     $unknownicon = file_extension_icon('', $size);
+    $phpicon = file_extension_icon('.php', $size); // Exception for php files.
 
     // We do not want to return those icon types, the module icon is more appropriate.
-    if ($icon === $unknownicon || $icon === $htmlicon) {
+    if ($icon === $unknownicon || $icon === $htmlicon || $icon === $phpicon) {
         return null;
     }
 
