@@ -102,8 +102,9 @@ class mod_wiki_external extends external_api {
 
                 $viewablefields = [];
                 if (has_capability('mod/wiki:viewpage', $context)) {
+                    $options = array('noclean' => true);
                     list($module['intro'], $module['introformat']) =
-                        external_format_text($wiki->intro, $wiki->introformat, $context->id, 'mod_wiki', 'intro', null);
+                        external_format_text($wiki->intro, $wiki->introformat, $context->id, 'mod_wiki', 'intro', null, $options);
                     $module['introfiles'] = external_util::get_area_files($context->id, 'mod_wiki', 'intro', false, false);
 
                     $viewablefields = array('firstpagetitle', 'wikimode', 'defaultformat', 'forceformat', 'editbegin', 'editend',
@@ -466,15 +467,17 @@ class mod_wiki_external extends external_api {
             throw new moodle_exception('cannotviewpage', 'wiki');
         } else if ($subwiki->id != -1) {
 
-            // Set sort param.
             $options = $params['options'];
-            if (!empty($options['sortby'])) {
-                if ($options['sortdirection'] != 'ASC' && $options['sortdirection'] != 'DESC') {
-                    // Invalid sort direction. Use default.
-                    $options['sortdirection'] = 'ASC';
-                }
-                $sort = $options['sortby'] . ' ' . $options['sortdirection'];
-            }
+
+            // Set sort param.
+            $sort = get_safe_orderby([
+                'id' => 'id',
+                'title' => 'title',
+                'timecreated' => 'timecreated',
+                'timemodified' => 'timemodified',
+                'pageviews' => 'pageviews',
+                'default' => 'title',
+            ], $options['sortby'], $options['sortdirection'], false);
 
             $pages = wiki_get_page_list($subwiki->id, $sort);
             $caneditpages = wiki_user_can_edit($subwiki);
@@ -511,10 +514,14 @@ class mod_wiki_external extends external_api {
                     $retpage['contentformat'] = $contentformat;
                 } else {
                     // Return the size of the content.
-                    if (function_exists('mb_strlen') && ((int)ini_get('mbstring.func_overload') & 2)) {
-                        $retpage['contentsize'] = mb_strlen($cachedcontent, '8bit');
-                    } else {
-                        $retpage['contentsize'] = strlen($cachedcontent);
+                    $retpage['contentsize'] = strlen($cachedcontent);
+                    // TODO: Remove this block of code once PHP 8.0 is the min version supported.
+                    // For PHP < 8.0, if strlen() was overloaded, calculate
+                    // the bytes using mb_strlen(..., '8bit').
+                    if (PHP_VERSION_ID < 80000) {
+                        if (function_exists('mb_strlen') && ((int)ini_get('mbstring.func_overload') & 2)) {
+                            $retpage['contentsize'] = mb_strlen($cachedcontent, '8bit');
+                        }
                     }
                 }
 
