@@ -14,35 +14,36 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
-/**
- * Unit tests for the multiple choice, single response question definition classes.
- *
- * @package   qtype_multichoice
- * @copyright 2009 The Open University
- * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
+namespace qtype_multichoice;
+
+use qtype_multichoice_multi_question;
+use question_answer;
+use question_attempt_step;
+use question_bank;
+use question_classified_response;
+use question_state;
 
 defined('MOODLE_INTERNAL') || die();
 
 global $CFG;
 require_once($CFG->dirroot . '/question/engine/tests/helpers.php');
 
-
 /**
  * Unit tests for the multiple choice, single response question definition class.
  *
+ * @package   qtype_multichoice
  * @copyright 2009 The Open University
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class qtype_multichoice_single_question_test extends advanced_testcase {
+class question_single_test extends \advanced_testcase {
 
     public function test_get_expected_data() {
-        $question = test_question_maker::make_a_multichoice_single_question();
+        $question = \test_question_maker::make_a_multichoice_single_question();
         $this->assertEquals(array('answer' => PARAM_INT), $question->get_expected_data());
     }
 
     public function test_is_complete_response() {
-        $question = test_question_maker::make_a_multichoice_single_question();
+        $question = \test_question_maker::make_a_multichoice_single_question();
 
         $this->assertFalse($question->is_complete_response(array()));
         $this->assertTrue($question->is_complete_response(array('answer' => '0')));
@@ -52,7 +53,7 @@ class qtype_multichoice_single_question_test extends advanced_testcase {
     }
 
     public function test_is_gradable_response() {
-        $question = test_question_maker::make_a_multichoice_single_question();
+        $question = \test_question_maker::make_a_multichoice_single_question();
 
         $this->assertFalse($question->is_gradable_response(array()));
         $this->assertTrue($question->is_gradable_response(array('answer' => '0')));
@@ -61,7 +62,7 @@ class qtype_multichoice_single_question_test extends advanced_testcase {
     }
 
     public function test_is_same_response() {
-        $question = test_question_maker::make_a_multichoice_single_question();
+        $question = \test_question_maker::make_a_multichoice_single_question();
         $question->start_attempt(new question_attempt_step(), 1);
 
         $this->assertTrue($question->is_same_response(
@@ -102,7 +103,7 @@ class qtype_multichoice_single_question_test extends advanced_testcase {
     }
 
     public function test_grading() {
-        $question = test_question_maker::make_a_multichoice_single_question();
+        $question = \test_question_maker::make_a_multichoice_single_question();
         $question->start_attempt(new question_attempt_step(), 1);
 
         $this->assertEquals(array(1, question_state::$gradedright),
@@ -116,7 +117,7 @@ class qtype_multichoice_single_question_test extends advanced_testcase {
     public function test_grading_rounding_three_right() {
         question_bank::load_question_definition_classes('multichoice');
         $mc = new qtype_multichoice_multi_question();
-        test_question_maker::initialise_a_question($mc);
+        \test_question_maker::initialise_a_question($mc);
         $mc->name = 'Odd numbers';
         $mc->questiontext = 'Which are the odd numbers?';
         $mc->generalfeedback = '1, 3 and 5 are the odd numbers.';
@@ -125,7 +126,7 @@ class qtype_multichoice_single_question_test extends advanced_testcase {
         $mc->answernumbering = 'abc';
         $mc->showstandardinstruction = 0;
 
-        test_question_maker::set_standard_combined_feedback_fields($mc);
+        \test_question_maker::set_standard_combined_feedback_fields($mc);
 
         $mc->answers = array(
             11 => new question_answer(11, '1', 0.3333333, '', FORMAT_HTML),
@@ -139,32 +140,32 @@ class qtype_multichoice_single_question_test extends advanced_testcase {
         $mc->start_attempt(new question_attempt_step(), 1);
 
         list($grade, $state) = $mc->grade_response($mc->prepare_simulated_post_data(array('1' => '1', '3' => '1', '5' => '1')));
-        $this->assertEquals(1, $grade, '', 0.000001);
+        $this->assertEqualsWithDelta(1, $grade, 0.000001);
         $this->assertEquals(question_state::$gradedright, $state);
     }
 
     public function test_get_correct_response() {
-        $question = test_question_maker::make_a_multichoice_single_question();
+        $question = \test_question_maker::make_a_multichoice_single_question();
         $question->start_attempt(new question_attempt_step(), 1);
 
         $this->assertEquals($question->prepare_simulated_post_data(array('answer' => 'A')), $question->get_correct_response());
     }
 
     public function test_summarise_response() {
-        $mc = test_question_maker::make_a_multichoice_single_question();
+        $mc = \test_question_maker::make_a_multichoice_single_question();
         $mc->start_attempt(new question_attempt_step(), 1);
 
         $summary = $mc->summarise_response($mc->prepare_simulated_post_data(array('answer' => 'A')),
-                                            test_question_maker::get_a_qa($mc));
+                                            \test_question_maker::get_a_qa($mc));
 
         $this->assertEquals('A', $summary);
 
-        $this->assertNull($mc->summarise_response(array(), test_question_maker::get_a_qa($mc)));
-        $this->assertNull($mc->summarise_response(array('answer' => '-1'), test_question_maker::get_a_qa($mc)));
+        $this->assertNull($mc->summarise_response(array(), \test_question_maker::get_a_qa($mc)));
+        $this->assertNull($mc->summarise_response(array('answer' => '-1'), \test_question_maker::get_a_qa($mc)));
     }
 
     public function test_classify_response() {
-        $mc = test_question_maker::make_a_multichoice_single_question();
+        $mc = \test_question_maker::make_a_multichoice_single_question();
         $mc->start_attempt(new question_attempt_step(), 1);
 
         $this->assertEquals(array($mc->id => new question_classified_response(14, 'B', -0.3333333)),
@@ -180,7 +181,7 @@ class qtype_multichoice_single_question_test extends advanced_testcase {
     }
 
     public function test_make_html_inline() {
-        $mc = test_question_maker::make_a_multichoice_single_question();
+        $mc = \test_question_maker::make_a_multichoice_single_question();
         $this->assertEquals('Frog', $mc->make_html_inline('<p>Frog</p>'));
         $this->assertEquals('Frog<br />Toad', $mc->make_html_inline("<p>Frog</p>\n<p>Toad</p>"));
         $this->assertEquals('<img src="http://example.com/pic.png" alt="Graph" />',
@@ -194,7 +195,7 @@ class qtype_multichoice_single_question_test extends advanced_testcase {
     }
 
     public function test_simulated_post_data() {
-        $mc = test_question_maker::make_a_multichoice_single_question();
+        $mc = \test_question_maker::make_a_multichoice_single_question();
         $mc->shuffleanswers = false;
         $mc->answers[13]->answer = '<p>A</p>';
         $mc->answers[14]->answer = '<p>B</p>';
