@@ -24,6 +24,12 @@
  * @since Moodle 2.4
  */
 
+namespace core_user;
+
+use core_files_external;
+use core_user_external;
+use externallib_advanced_testcase;
+
 defined('MOODLE_INTERNAL') || die();
 
 global $CFG;
@@ -32,7 +38,7 @@ require_once($CFG->dirroot . '/webservice/tests/helpers.php');
 require_once($CFG->dirroot . '/user/externallib.php');
 require_once($CFG->dirroot . '/files/externallib.php');
 
-class core_user_externallib_testcase extends externallib_advanced_testcase {
+class externallib_test extends externallib_advanced_testcase {
 
     /**
      * Test get_users
@@ -74,7 +80,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $generatedusers[$user1->id] = $user1;
         $generatedusers[$user2->id] = $user2;
 
-        $context = context_course::instance($course->id);
+        $context = \context_course::instance($course->id);
         $roleid = $this->assignUserCapability('moodle/user:viewdetails', $context->id);
 
         // Enrol the users in the course.
@@ -94,7 +100,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $result = core_user_external::get_users($searchparams);
 
         // We need to execute the return values cleaning process to simulate the web service server
-        $result = external_api::clean_returnvalue(core_user_external::get_users_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::get_users_returns(), $result);
 
         // Check we retrieve the good total number of enrolled users + no error on capability.
         $expectedreturnedusers = 1;
@@ -162,9 +168,9 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
             // Call the external function.
             $result = core_user_external::get_users($searchparams);
             $this->fail('Expecting \'keyalreadyset\' moodle_exception to be thrown.');
-        } catch (moodle_exception $e) {
+        } catch (\moodle_exception $e) {
             $this->assertEquals('keyalreadyset', $e->errorcode);
-        } catch (Exception $e) {
+        } catch (\Exception $e) {
             $this->fail('Expecting \'keyalreadyset\' moodle_exception to be thrown.');
         }
     }
@@ -207,7 +213,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $generatedusers[$user1->id] = $user1;
         $generatedusers[$user2->id] = $user2;
 
-        $context = context_course::instance($course->id);
+        $context = \context_course::instance($course->id);
         $roleid = $this->assignUserCapability('moodle/user:viewdetails', $context->id);
 
         // Enrol the users in the course.
@@ -225,7 +231,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
             // Call the external function.
             $returnedusers = core_user_external::get_users_by_field($fieldtosearch,
                         array($USER->{$fieldtosearch}, $user1->{$fieldtosearch}, $user2->{$fieldtosearch}));
-            $returnedusers = external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
+            $returnedusers = \external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
 
             // Expected result differ following the searched field
             // Admin user in the PHPunit framework doesn't have an idnumber.
@@ -292,7 +298,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         // Call the external function.
         $returnedusers = core_user_external::get_users_by_field('username',
                     array($USER->username, $user1->username, $user2->username));
-        $returnedusers = external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
+        $returnedusers = \external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
 
         // Only the own $USER username should be returned
         $this->assertEquals(1, count($returnedusers));
@@ -303,7 +309,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         // Call the external function.
         $returnedusers = core_user_external::get_users_by_field('username',
             array($USER->username, $user1->username, $user2->username));
-        $returnedusers = external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
+        $returnedusers = \external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
 
         // Only the own $USER username should be returned still.
         $this->assertEquals(1, count($returnedusers));
@@ -314,7 +320,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
 
         $this->resetAfterTest(true);
 
-        $return = new stdClass();
+        $return = new \stdClass();
 
         // Create the course and fetch its context.
         $return->course = self::getDataGenerator()->create_course();
@@ -342,7 +348,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         }
         $return->user2 = self::getDataGenerator()->create_user();
 
-        $context = context_course::instance($return->course->id);
+        $context = \context_course::instance($return->course->id);
         $return->roleid = $this->assignUserCapability($capability, $context->id);
 
         // Enrol the users in the course.
@@ -368,7 +374,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
                     array('userid' => $USER->id, 'courseid' => $data->course->id)));
 
         // We need to execute the return values cleaning process to simulate the web service server.
-        $enrolledusers = external_api::clean_returnvalue(core_user_external::get_course_user_profiles_returns(), $enrolledusers);
+        $enrolledusers = \external_api::clean_returnvalue(core_user_external::get_course_user_profiles_returns(), $enrolledusers);
 
         // Check we retrieve the good total number of enrolled users + no error on capability.
         $this->assertEquals(1, count($enrolledusers));
@@ -392,7 +398,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
             array('userid' => $data->user1->id, 'courseid' => $data->course->id)));
 
         // We need to execute the return values cleaning process to simulate the web service server.
-        $enrolledusers = external_api::clean_returnvalue(core_user_external::get_course_user_profiles_returns(), $enrolledusers);
+        $enrolledusers = \external_api::clean_returnvalue(core_user_external::get_course_user_profiles_returns(), $enrolledusers);
 
         foreach($enrolledusers as $enrolleduser) {
             if ($enrolleduser['username'] == $data->user1->username) {
@@ -462,7 +468,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
             'auth' => 'oauth2'
         );
 
-        $context = context_system::instance();
+        $context = \context_system::instance();
         $roleid = $this->assignUserCapability('moodle/user:create', $context->id);
         $this->assignUserCapability('moodle/user:editprofile', $context->id, $roleid);
 
@@ -470,7 +476,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $createdusers = core_user_external::create_users(array($user1, $user2));
 
         // We need to execute the return values cleaning process to simulate the web service server.
-        $createdusers = external_api::clean_returnvalue(core_user_external::create_users_returns(), $createdusers);
+        $createdusers = \external_api::clean_returnvalue(core_user_external::create_users_returns(), $createdusers);
 
         // Check we retrieve the good total number of created users + no error on capability.
         $this->assertCount(2, $createdusers);
@@ -483,8 +489,8 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
                 $this->assertEquals('atto', get_user_preferences('htmleditor', null, $dbuser));
                 $this->assertEquals(null, get_user_preferences('invalidpreference', null, $dbuser));
                 // Confirm user interests have been saved.
-                $interests = core_tag_tag::get_item_tags_array('core', 'user', $createduser['id'],
-                        core_tag_tag::BOTH_STANDARD_AND_NOT, 0, false);
+                $interests = \core_tag_tag::get_item_tags_array('core', 'user', $createduser['id'],
+                        \core_tag_tag::BOTH_STANDARD_AND_NOT, 0, false);
                 // There should be 3 user interests.
                 $this->assertCount(3, $interests);
 
@@ -530,7 +536,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         ];
 
         // This should throw an exception because either password or createpassword param must be passed for auth_manual.
-        $this->expectException(invalid_parameter_exception::class);
+        $this->expectException(\invalid_parameter_exception::class);
         core_user_external::create_users([$user]);
     }
 
@@ -588,7 +594,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
 
         if (!$sameemailallowed) {
             // This should throw an exception when $CFG->allowaccountssameemail is empty.
-            $this->expectException(invalid_parameter_exception::class);
+            $this->expectException(\invalid_parameter_exception::class);
         }
 
         // Create our users.
@@ -692,7 +698,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $this->assertEquals(2, $DB->count_records_select('user', 'deleted = 0 AND (id = :userid1 OR id = :userid2)',
                 array('userid1' => $user1->id, 'userid2' => $user2->id)));
 
-        $context = context_system::instance();
+        $context = \context_system::instance();
         $roleid = $this->assignUserCapability('moodle/user:delete', $context->id);
 
         // Call the external function.
@@ -719,7 +725,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $wsuser = self::getDataGenerator()->create_user();
         self::setUser($wsuser);
 
-        $context = context_user::instance($USER->id);
+        $context = \context_user::instance($USER->id);
         $contextid = $context->id;
         $filename = "reddot.png";
         $filecontent = "iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38"
@@ -728,7 +734,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         // Call the files api to create a file.
         $draftfile = core_files_external::upload($contextid, 'user', 'draft', 0, '/',
                 $filename, $filecontent, null, null);
-        $draftfile = external_api::clean_returnvalue(core_files_external::upload_returns(), $draftfile);
+        $draftfile = \external_api::clean_returnvalue(core_files_external::upload_returns(), $draftfile);
 
         $draftid = $draftfile['itemid'];
 
@@ -765,7 +771,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
             'interests' => 'badminton, basketball, cooking,  '
         );
 
-        $context = context_system::instance();
+        $context = \context_system::instance();
         $roleid = $this->assignUserCapability('moodle/user:update', $context->id);
         $this->assignUserCapability('moodle/user:editprofile', $context->id, $roleid);
 
@@ -808,7 +814,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $this->assertEquals(null, get_user_preferences('invalidpreference', null, $dbuser));
 
         // Confirm user interests have been saved.
-        $interests = core_tag_tag::get_item_tags_array('core', 'user', $user1['id'], core_tag_tag::BOTH_STANDARD_AND_NOT, 0, false);
+        $interests = \core_tag_tag::get_item_tags_array('core', 'user', $user1['id'], \core_tag_tag::BOTH_STANDARD_AND_NOT, 0, false);
         // There should be 3 user interests.
         $this->assertCount(3, $interests);
 
@@ -924,10 +930,10 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
 
         $this->resetAfterTest(true);
 
-        $context = context_system::instance();
+        $context = \context_system::instance();
         $roleid = $this->assignUserCapability('moodle/user:manageownfiles', $context->id);
 
-        $context = context_user::instance($USER->id);
+        $context = \context_user::instance($USER->id);
         $contextid = $context->id;
         $component = "user";
         $filearea = "draft";
@@ -942,7 +948,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         // Call the files api to create a file.
         $draftfile = core_files_external::upload($contextid, $component, $filearea, $itemid, $filepath,
                                                  $filename, $filecontent, $contextlevel, $instanceid);
-        $draftfile = external_api::clean_returnvalue(core_files_external::upload_returns(), $draftfile);
+        $draftfile = \external_api::clean_returnvalue(core_files_external::upload_returns(), $draftfile);
 
         $draftid = $draftfile['itemid'];
         // Make sure the file was created.
@@ -992,14 +998,14 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $warnings = core_user_external::add_user_device($device['appid'], $device['name'], $device['model'], $device['platform'],
                                                         $device['version'], $device['pushid'], $device['uuid']);
         // We need to execute the return values cleaning process to simulate the web service server.
-        $warnings = external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
+        $warnings = \external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
         $this->assertCount(1, $warnings);
 
         // Test update an existing device.
         $device['pushid'] = 'different than before';
         $warnings = core_user_external::add_user_device($device['appid'], $device['name'], $device['model'], $device['platform'],
                                                         $device['version'], $device['pushid'], $device['uuid']);
-        $warnings = external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
+        $warnings = \external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
 
         $this->assertEquals(1, $DB->count_records('user_devices'));
         $updated = $DB->get_record('user_devices', array('pushid' => $device['pushid']));
@@ -1010,7 +1016,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $device['pushid'] = 'new different than before';
         $warnings = core_user_external::add_user_device($device['appid'], $device['name'], $device['model'], $device['platform'],
                                                         $device['version'], $device['pushid'], $device['uuid']);
-        $warnings = external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
+        $warnings = \external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
         $this->assertEquals(2, $DB->count_records('user_devices'));
     }
 
@@ -1048,24 +1054,24 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
 
         // Try to remove a device that does not exist.
         $result = core_user_external::remove_user_device('1234567890');
-        $result = external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
         $this->assertFalse($result['removed']);
         $this->assertCount(1, $result['warnings']);
 
         // Try to remove a device that does not exist for an existing app.
         $result = core_user_external::remove_user_device('1234567890', $device['appid']);
-        $result = external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
         $this->assertFalse($result['removed']);
         $this->assertCount(1, $result['warnings']);
 
         // Remove an existing device for an existing app. This will remove one of the two devices.
         $result = core_user_external::remove_user_device($device['uuid'], $device['appid']);
-        $result = external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
         $this->assertTrue($result['removed']);
 
         // Remove all the devices. This must remove the remaining device.
         $result = core_user_external::remove_user_device($device['uuid']);
-        $result = external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
         $this->assertTrue($result['removed']);
     }
 
@@ -1082,7 +1088,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $this->setUser($user);
 
         $result = core_user_external::get_user_preferences();
-        $result = external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         // Expect 3, _lastloaded is always returned.
         $this->assertCount(3, $result['preferences']);
@@ -1097,14 +1103,14 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
 
         // Retrieve just one preference.
         $result = core_user_external::get_user_preferences('some_random_text');
-        $result = external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(1, $result['preferences']);
         $this->assertEquals('text', $result['preferences'][0]['value']);
 
         // Retrieve non-existent preference.
         $result = core_user_external::get_user_preferences('non_existent');
-        $result = external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(1, $result['preferences']);
         $this->assertEquals(null, $result['preferences'][0]['value']);
@@ -1112,7 +1118,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         // Check that as admin we can retrieve all the preferences for any user.
         $this->setAdminUser();
         $result = core_user_external::get_user_preferences('', $user->id);
-        $result = external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(3, $result['preferences']);
 
@@ -1143,7 +1149,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $user = self::getDataGenerator()->create_user();
         self::setUser($user);
 
-        $context = context_user::instance($USER->id);
+        $context = \context_user::instance($USER->id);
         $contextid = $context->id;
         $filename = "reddot.png";
         $filecontent = "iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38"
@@ -1155,7 +1161,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
 
         // Change user profile image.
         $result = core_user_external::update_picture($draftid);
-        $result = external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
         $picture = $DB->get_field('user', 'picture', array('id' => $user->id));
         // The new revision is in the url for the user.
         $this->assertStringContainsString($picture, $result['profileimageurl']);
@@ -1164,7 +1170,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
 
         // Delete image.
         $result = core_user_external::update_picture(0, true);
-        $result = external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
         $picture = $DB->get_field('user', 'picture', array('id' => $user->id));
         // No picture.
         $this->assertEquals(0, $picture);
@@ -1172,13 +1178,13 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         // Add again the user profile image (as admin).
         $this->setAdminUser();
 
-        $context = context_user::instance($USER->id);
+        $context = \context_user::instance($USER->id);
         $admincontextid = $context->id;
         $draftfile = core_files_external::upload($admincontextid, 'user', 'draft', 0, '/', $filename, $filecontent, null, null);
         $draftid = $draftfile['itemid'];
 
         $result = core_user_external::update_picture($draftid, false, $user->id);
-        $result = external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
         // The new revision is in the url for the user.
         $picture = $DB->get_field('user', 'picture', array('id' => $user->id));
         $this->assertStringContainsString($picture, $result['profileimageurl']);
@@ -1224,7 +1230,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         );
 
         $result = core_user_external::set_user_preferences($preferences);
-        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(2, $result['saved']);
 
@@ -1255,7 +1261,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         );
 
         $result = core_user_external::set_user_preferences($preferences);
-        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
         $this->assertCount(1, $result['warnings']);
         $this->assertCount(0, $result['saved']);
         $this->assertEquals('nopermission', $result['warnings'][0]['warningcode']);
@@ -1280,7 +1286,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         );
 
         $result = core_user_external::set_user_preferences($preferences);
-        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
         $this->assertCount(1, $result['warnings']);
         $this->assertCount(0, $result['saved']);
         $this->assertEquals('invaliduser', $result['warnings'][0]['warningcode']);
@@ -1305,7 +1311,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         );
 
         $result = core_user_external::set_user_preferences($preferences);
-        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(1, $result['saved']);
         // Cleaned valud of the preference was saved.
@@ -1359,7 +1365,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         );
 
         $result = core_user_external::set_user_preferences($preferences);
-        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(1, $result['saved']);
 
@@ -1385,7 +1391,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
 
         // Site policy not set.
         $result = core_user_external::agree_site_policy();
-        $result = external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
         $this->assertFalse($result['status']);
         $this->assertCount(1, $result['warnings']);
         $this->assertEquals('nositepolicy', $result['warnings'][0]['warningcode']);
@@ -1395,7 +1401,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $this->assertEquals(0, $USER->policyagreed);
 
         $result = core_user_external::agree_site_policy();
-        $result = external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
         $this->assertTrue($result['status']);
         $this->assertCount(0, $result['warnings']);
         $this->assertEquals(1, $USER->policyagreed);
@@ -1403,7 +1409,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
 
         // Try again, we should get a warning.
         $result = core_user_external::agree_site_policy();
-        $result = external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
         $this->assertFalse($result['status']);
         $this->assertCount(1, $result['warnings']);
         $this->assertEquals('alreadyagreed', $result['warnings'][0]['warningcode']);
@@ -1417,9 +1423,9 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         try {
             $result = core_user_external::agree_site_policy();
             $this->fail('Expecting \'usernotfullysetup\' moodle_exception to be thrown');
-        } catch (moodle_exception $e) {
+        } catch (\moodle_exception $e) {
             $this->assertEquals('usernotfullysetup', $e->errorcode);
-        } catch (Exception $e) {
+        } catch (\Exception $e) {
             $this->fail('Expecting \'usernotfullysetup\' moodle_exception to be thrown.');
         }
     }
@@ -1432,7 +1438,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $this->resetAfterTest(true);
         $user = self::getDataGenerator()->create_user();
         $this->setUser($user);
-        $usercontext = context_user::instance($user->id);
+        $usercontext = \context_user::instance($user->id);
 
         $filerecord = array(
             'contextid' => $usercontext->id,
@@ -1448,7 +1454,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
 
         // Get my private files information.
         $result = core_user_external::get_private_files_info();
-        $result = external_api::clean_returnvalue(core_user_external::get_private_files_info_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::get_private_files_info_returns(), $result);
         $this->assertEquals(1, $result['filecount']);
         $this->assertEquals($file->get_filesize(), $result['filesize']);
         $this->assertEquals(0, $result['foldercount']);
@@ -1457,7 +1463,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         // As admin, get user information.
         $this->setAdminUser();
         $result = core_user_external::get_private_files_info($user->id);
-        $result = external_api::clean_returnvalue(core_user_external::get_private_files_info_returns(), $result);
+        $result = \external_api::clean_returnvalue(core_user_external::get_private_files_info_returns(), $result);
         $this->assertEquals(1, $result['filecount']);
         $this->assertEquals($file->get_filesize(), $result['filesize']);
         $this->assertEquals(0, $result['foldercount']);
@@ -1534,7 +1540,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $CFG->maxusersperpage = 3;
 
         $result = \core_user\external\search_identity::execute('Lastt');
-        $result = external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
+        $result = \external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
 
         $this->assertEquals(2, count($result['list']));
         $this->assertEquals(3, $result['maxusersperpage']);
@@ -1551,7 +1557,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $CFG->maxusersperpage = 2;
 
         $result = \core_user\external\search_identity::execute('Firstt');
-        $result = external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
+        $result = \external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
 
         $this->assertEquals(2, count($result['list']));
         $this->assertEquals(2, $result['maxusersperpage']);
@@ -1566,7 +1572,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         $CFG->maxusersperpage = 2;
 
         $result = \core_user\external\search_identity::execute('City One');
-        $result = external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
+        $result = \external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
 
         $this->assertEquals(0, count($result['list']));
         $this->assertEquals(2, $result['maxusersperpage']);
@@ -1581,7 +1587,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
         }
 
         $result = \core_user\external\search_identity::execute('City One');
-        $result = external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
+        $result = \external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
 
         $this->assertEquals(2, count($result['list']));
         $this->assertEquals(2, $result['maxusersperpage']);
@@ -1612,7 +1618,7 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
             '<ruby>lastname firstname <rp>(</rp><rt>lastnamephonetic firstnamephonetic</rt><rp>)</rp></ruby>';
 
         $result = \core_user\external\search_identity::execute('Ak');
-        $result = external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
+        $result = \external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
 
         $this->assertEquals(1, count($result['list']));
         $this->assertEquals(3, $result['maxusersperpage']);
@@ -1623,4 +1629,36 @@ class core_user_externallib_testcase extends externallib_advanced_testcase {
             $this->assertEquals('email', $user['extrafields'][0]['name']);
         }
     }
+
+    /**
+     * Test verifying that update_user_preferences prevents changes to the default homepage for other users.
+     */
+    public function test_update_user_preferences_homepage_permission_callback() {
+        global $DB;
+        $this->resetAfterTest();
+
+        $user = self::getDataGenerator()->create_user();
+        $this->setUser($user);
+        $adminuser = get_admin();
+
+        // Allow user selection of the default homepage via preferences.
+        set_config('defaulthomepage', HOMEPAGE_USER);
+
+        // Try to save another user's home page preference which uses the permissioncallback.
+        $preferences = [
+            [
+                'name' => 'user_home_page_preference',
+                'value' => '3',
+                'userid' => $adminuser->id,
+            ]
+        ];
+        $result = core_user_external::set_user_preferences($preferences);
+        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $this->assertCount(1, $result['warnings']);
+        $this->assertCount(0, $result['saved']);
+
+        // Verify no change to the preference, checking from DB to avoid cache.
+        $this->assertEquals(null, $DB->get_field('user_preferences', 'value',
+            ['userid' => $adminuser->id, 'name' => 'user_home_page_preference']));
+    }
 }
